/****************************************************************************
** Meta object code from reading C++ file 'TagTemplate.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../merkaartor/src/TagTemplate/TagTemplate.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'TagTemplate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17TagTemplateWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplateWidget::qt_create_metaobjectdata<qt_meta_tag_ZN17TagTemplateWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplateWidget",
        "tagCleared",
        "",
        "k",
        "tagChanged",
        "v"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'tagCleared'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'tagChanged'
        QtMocHelpers::SignalData<void(QString, QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplateWidget, qt_meta_tag_ZN17TagTemplateWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplateWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TagTemplateWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TagTemplateWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17TagTemplateWidgetE_t>.metaTypes,
    nullptr
} };

void TagTemplateWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplateWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tagCleared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->tagChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagTemplateWidget::*)(QString )>(_a, &TagTemplateWidget::tagCleared, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagTemplateWidget::*)(QString , QString )>(_a, &TagTemplateWidget::tagChanged, 1))
            return;
    }
}

const QMetaObject *TagTemplateWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplateWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17TagTemplateWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TagTemplateWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void TagTemplateWidget::tagCleared(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TagTemplateWidget::tagChanged(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN22TagTemplateWidgetValueE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplateWidgetValue::qt_create_metaobjectdata<qt_meta_tag_ZN22TagTemplateWidgetValueE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplateWidgetValue"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplateWidgetValue, qt_meta_tag_ZN22TagTemplateWidgetValueE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplateWidgetValue::staticMetaObject = { {
    QMetaObject::SuperData::link<TagTemplateWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetValueE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetValueE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22TagTemplateWidgetValueE_t>.metaTypes,
    nullptr
} };

void TagTemplateWidgetValue::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplateWidgetValue *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TagTemplateWidgetValue::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplateWidgetValue::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetValueE_t>.strings))
        return static_cast<void*>(this);
    return TagTemplateWidget::qt_metacast(_clname);
}

int TagTemplateWidgetValue::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TagTemplateWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN22TagTemplateWidgetComboE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplateWidgetCombo::qt_create_metaobjectdata<qt_meta_tag_ZN22TagTemplateWidgetComboE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplateWidgetCombo",
        "on_combo_activated",
        "",
        "idx"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'on_combo_activated'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplateWidgetCombo, qt_meta_tag_ZN22TagTemplateWidgetComboE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplateWidgetCombo::staticMetaObject = { {
    QMetaObject::SuperData::link<TagTemplateWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetComboE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetComboE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22TagTemplateWidgetComboE_t>.metaTypes,
    nullptr
} };

void TagTemplateWidgetCombo::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplateWidgetCombo *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->on_combo_activated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *TagTemplateWidgetCombo::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplateWidgetCombo::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetComboE_t>.strings))
        return static_cast<void*>(this);
    return TagTemplateWidget::qt_metacast(_clname);
}

int TagTemplateWidgetCombo::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TagTemplateWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN22TagTemplateWidgetYesnoE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplateWidgetYesno::qt_create_metaobjectdata<qt_meta_tag_ZN22TagTemplateWidgetYesnoE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplateWidgetYesno",
        "on_checkbox_stateChanged",
        "",
        "state"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'on_checkbox_stateChanged'
        QtMocHelpers::SlotData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplateWidgetYesno, qt_meta_tag_ZN22TagTemplateWidgetYesnoE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplateWidgetYesno::staticMetaObject = { {
    QMetaObject::SuperData::link<TagTemplateWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetYesnoE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetYesnoE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN22TagTemplateWidgetYesnoE_t>.metaTypes,
    nullptr
} };

void TagTemplateWidgetYesno::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplateWidgetYesno *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->on_checkbox_stateChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        default: ;
        }
    }
}

const QMetaObject *TagTemplateWidgetYesno::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplateWidgetYesno::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN22TagTemplateWidgetYesnoE_t>.strings))
        return static_cast<void*>(this);
    return TagTemplateWidget::qt_metacast(_clname);
}

int TagTemplateWidgetYesno::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TagTemplateWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN25TagTemplateWidgetConstantE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplateWidgetConstant::qt_create_metaobjectdata<qt_meta_tag_ZN25TagTemplateWidgetConstantE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplateWidgetConstant"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplateWidgetConstant, qt_meta_tag_ZN25TagTemplateWidgetConstantE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplateWidgetConstant::staticMetaObject = { {
    QMetaObject::SuperData::link<TagTemplateWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25TagTemplateWidgetConstantE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25TagTemplateWidgetConstantE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN25TagTemplateWidgetConstantE_t>.metaTypes,
    nullptr
} };

void TagTemplateWidgetConstant::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplateWidgetConstant *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TagTemplateWidgetConstant::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplateWidgetConstant::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN25TagTemplateWidgetConstantE_t>.strings))
        return static_cast<void*>(this);
    return TagTemplateWidget::qt_metacast(_clname);
}

int TagTemplateWidgetConstant::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TagTemplateWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN21TagTemplateWidgetEditE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplateWidgetEdit::qt_create_metaobjectdata<qt_meta_tag_ZN21TagTemplateWidgetEditE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplateWidgetEdit",
        "on_editingFinished",
        ""
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'on_editingFinished'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplateWidgetEdit, qt_meta_tag_ZN21TagTemplateWidgetEditE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplateWidgetEdit::staticMetaObject = { {
    QMetaObject::SuperData::link<TagTemplateWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21TagTemplateWidgetEditE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21TagTemplateWidgetEditE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN21TagTemplateWidgetEditE_t>.metaTypes,
    nullptr
} };

void TagTemplateWidgetEdit::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplateWidgetEdit *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->on_editingFinished(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *TagTemplateWidgetEdit::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplateWidgetEdit::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN21TagTemplateWidgetEditE_t>.strings))
        return static_cast<void*>(this);
    return TagTemplateWidget::qt_metacast(_clname);
}

int TagTemplateWidgetEdit::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = TagTemplateWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 1)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 1)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 1;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN11TagTemplateE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplate::qt_create_metaobjectdata<qt_meta_tag_ZN11TagTemplateE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplate",
        "tagCleared",
        "",
        "k",
        "tagChanged",
        "v",
        "on_tag_cleared",
        "on_tag_changed",
        "vaL"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'tagCleared'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'tagChanged'
        QtMocHelpers::SignalData<void(QString, QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 5 },
        }}),
        // Slot 'on_tag_cleared'
        QtMocHelpers::SlotData<void(QString)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'on_tag_changed'
        QtMocHelpers::SlotData<void(QString, QString)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 8 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplate, qt_meta_tag_ZN11TagTemplateE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplate::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TagTemplateE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TagTemplateE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11TagTemplateE_t>.metaTypes,
    nullptr
} };

void TagTemplate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplate *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tagCleared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->tagChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->on_tag_cleared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->on_tag_changed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagTemplate::*)(QString )>(_a, &TagTemplate::tagCleared, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagTemplate::*)(QString , QString )>(_a, &TagTemplate::tagChanged, 1))
            return;
    }
}

const QMetaObject *TagTemplate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TagTemplateE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TagTemplate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void TagTemplate::tagCleared(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TagTemplate::tagChanged(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
namespace {
struct qt_meta_tag_ZN12TagTemplatesE_t {};
} // unnamed namespace

template <> constexpr inline auto TagTemplates::qt_create_metaobjectdata<qt_meta_tag_ZN12TagTemplatesE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagTemplates",
        "tagCleared",
        "",
        "k",
        "tagChanged",
        "v",
        "templateChanged",
        "TagTemplate*",
        "aNewTemplate",
        "on_combo_activated",
        "idx",
        "on_tag_cleared",
        "on_tag_changed"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'tagCleared'
        QtMocHelpers::SignalData<void(QString)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'tagChanged'
        QtMocHelpers::SignalData<void(QString, QString)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 5 },
        }}),
        // Signal 'templateChanged'
        QtMocHelpers::SignalData<void(TagTemplate *)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'on_combo_activated'
        QtMocHelpers::SlotData<void(int)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 10 },
        }}),
        // Slot 'on_tag_cleared'
        QtMocHelpers::SlotData<void(QString)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'on_tag_changed'
        QtMocHelpers::SlotData<void(QString, QString)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QString, 5 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagTemplates, qt_meta_tag_ZN12TagTemplatesE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagTemplates::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagTemplatesE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagTemplatesE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12TagTemplatesE_t>.metaTypes,
    nullptr
} };

void TagTemplates::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagTemplates *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->tagCleared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->tagChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 2: _t->templateChanged((*reinterpret_cast<std::add_pointer_t<TagTemplate*>>(_a[1]))); break;
        case 3: _t->on_combo_activated((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->on_tag_cleared((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->on_tag_changed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< TagTemplate* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagTemplates::*)(QString )>(_a, &TagTemplates::tagCleared, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagTemplates::*)(QString , QString )>(_a, &TagTemplates::tagChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagTemplates::*)(TagTemplate * )>(_a, &TagTemplates::templateChanged, 2))
            return;
    }
}

const QMetaObject *TagTemplates::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagTemplates::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12TagTemplatesE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TagTemplates::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void TagTemplates::tagCleared(QString _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TagTemplates::tagChanged(QString _t1, QString _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void TagTemplates::templateChanged(TagTemplate * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
