/********************************************************************************
** Form generated from reading UI file 'FilterEditDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTEREDITDIALOG_H
#define UI_FILTEREDITDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_FilterEditDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *edName;
    QLabel *label_2;
    QHBoxLayout *FilterLayout;
    QLineEdit *edFilter;
    QToolButton *btFilterHelper;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *FilterEditDialog)
    {
        if (FilterEditDialog->objectName().isEmpty())
            FilterEditDialog->setObjectName("FilterEditDialog");
        FilterEditDialog->resize(400, 165);
        verticalLayout = new QVBoxLayout(FilterEditDialog);
        verticalLayout->setObjectName("verticalLayout");
        label = new QLabel(FilterEditDialog);
        label->setObjectName("label");

        verticalLayout->addWidget(label);

        edName = new QLineEdit(FilterEditDialog);
        edName->setObjectName("edName");

        verticalLayout->addWidget(edName);

        label_2 = new QLabel(FilterEditDialog);
        label_2->setObjectName("label_2");

        verticalLayout->addWidget(label_2);

        FilterLayout = new QHBoxLayout();
        FilterLayout->setObjectName("FilterLayout");
        edFilter = new QLineEdit(FilterEditDialog);
        edFilter->setObjectName("edFilter");

        FilterLayout->addWidget(edFilter);

        btFilterHelper = new QToolButton(FilterEditDialog);
        btFilterHelper->setObjectName("btFilterHelper");

        FilterLayout->addWidget(btFilterHelper);


        verticalLayout->addLayout(FilterLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(FilterEditDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(FilterEditDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, FilterEditDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, FilterEditDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(FilterEditDialog);
    } // setupUi

    void retranslateUi(QDialog *FilterEditDialog)
    {
        FilterEditDialog->setWindowTitle(QCoreApplication::translate("FilterEditDialog", "Dialog", nullptr));
        label->setText(QCoreApplication::translate("FilterEditDialog", "Name", nullptr));
        label_2->setText(QCoreApplication::translate("FilterEditDialog", "Filter", nullptr));
        btFilterHelper->setText(QCoreApplication::translate("FilterEditDialog", "...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FilterEditDialog: public Ui_FilterEditDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTEREDITDIALOG_H
