/****************************************************************************
** Meta object code from reading C++ file 'LayerDock.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../merkaartor/src/Docks/LayerDock.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'LayerDock.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9LayerDockE_t {};
} // unnamed namespace

template <> constexpr inline auto LayerDock::qt_create_metaobjectdata<qt_meta_tag_ZN9LayerDockE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "LayerDock",
        "layersChanged",
        "",
        "adjustViewport",
        "layersClosed",
        "layersCleared",
        "layersProjection",
        "layerChanged",
        "LayerWidget*",
        "layerClosed",
        "Layer*",
        "layerCleared",
        "layerZoom",
        "layerProjection",
        "tabChanged",
        "idx",
        "tabContextMenuRequested",
        "QPoint",
        "pos",
        "TabShowAll",
        "TabHideAll",
        "showAllLayers",
        "hideAllLayers",
        "readonlyAllLayers",
        "readonlyNoneLayers",
        "closeLayers",
        "resetLayers"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'layersChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'layersClosed'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layersCleared'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'layersProjection'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'layerChanged'
        QtMocHelpers::SlotData<void(LayerWidget *, bool)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 8, 2 }, { QMetaType::Bool, 3 },
        }}),
        // Slot 'layerClosed'
        QtMocHelpers::SlotData<void(Layer *)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'layerCleared'
        QtMocHelpers::SlotData<void(Layer *)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'layerZoom'
        QtMocHelpers::SlotData<void(Layer *)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 2 },
        }}),
        // Slot 'layerProjection'
        QtMocHelpers::SlotData<void(const QString &)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 2 },
        }}),
        // Slot 'tabChanged'
        QtMocHelpers::SlotData<void(int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Slot 'tabContextMenuRequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(16, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 17, 18 },
        }}),
        // Slot 'TabShowAll'
        QtMocHelpers::SlotData<void(bool)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'TabHideAll'
        QtMocHelpers::SlotData<void(bool)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'showAllLayers'
        QtMocHelpers::SlotData<void(bool)>(21, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'hideAllLayers'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'readonlyAllLayers'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'readonlyNoneLayers'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'closeLayers'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'resetLayers'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LayerDock, qt_meta_tag_ZN9LayerDockE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject LayerDock::staticMetaObject = { {
    QMetaObject::SuperData::link<MDockAncestor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9LayerDockE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9LayerDockE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9LayerDockE_t>.metaTypes,
    nullptr
} };

void LayerDock::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LayerDock *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->layersChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->layersClosed(); break;
        case 2: _t->layersCleared(); break;
        case 3: _t->layersProjection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->layerChanged((*reinterpret_cast<std::add_pointer_t<LayerWidget*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 5: _t->layerClosed((*reinterpret_cast<std::add_pointer_t<Layer*>>(_a[1]))); break;
        case 6: _t->layerCleared((*reinterpret_cast<std::add_pointer_t<Layer*>>(_a[1]))); break;
        case 7: _t->layerZoom((*reinterpret_cast<std::add_pointer_t<Layer*>>(_a[1]))); break;
        case 8: _t->layerProjection((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->tabChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->tabContextMenuRequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 11: _t->TabShowAll((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->TabHideAll((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->showAllLayers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->hideAllLayers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->readonlyAllLayers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->readonlyNoneLayers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->closeLayers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->resetLayers(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LayerDock::*)(bool )>(_a, &LayerDock::layersChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LayerDock::*)()>(_a, &LayerDock::layersClosed, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LayerDock::*)()>(_a, &LayerDock::layersCleared, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LayerDock::*)(const QString & )>(_a, &LayerDock::layersProjection, 3))
            return;
    }
}

const QMetaObject *LayerDock::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LayerDock::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9LayerDockE_t>.strings))
        return static_cast<void*>(this);
    return MDockAncestor::qt_metacast(_clname);
}

int LayerDock::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = MDockAncestor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void LayerDock::layersChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void LayerDock::layersClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void LayerDock::layersCleared()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void LayerDock::layersProjection(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
