/********************************************************************************
** Form generated from reading UI file 'FilterPreferencesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FILTERPREFERENCESDIALOG_H
#define UI_FILTERPREFERENCESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_FilterPreferencesDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *grpFilters;
    QHBoxLayout *hboxLayout;
    QFrame *frame_2;
    QVBoxLayout *vboxLayout1;
    QLabel *label_10;
    QListWidget *lvFilters;
    QFrame *frame;
    QVBoxLayout *verticalLayout;
    QLabel *label_11;
    QLineEdit *edFilterName;
    QLabel *label_5;
    QHBoxLayout *horizontalLayout;
    QLineEdit *edFilterString;
    QToolButton *btFilterHelper;
    QSpacerItem *spacerItem;
    QPushButton *btApply;
    QPushButton *btAdd;
    QPushButton *btDel;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *FilterPreferencesDialog)
    {
        if (FilterPreferencesDialog->objectName().isEmpty())
            FilterPreferencesDialog->setObjectName("FilterPreferencesDialog");
        FilterPreferencesDialog->resize(640, 360);
        vboxLayout = new QVBoxLayout(FilterPreferencesDialog);
        vboxLayout->setObjectName("vboxLayout");
        grpFilters = new QGroupBox(FilterPreferencesDialog);
        grpFilters->setObjectName("grpFilters");
        hboxLayout = new QHBoxLayout(grpFilters);
        hboxLayout->setSpacing(1);
        hboxLayout->setContentsMargins(1, 1, 1, 1);
        hboxLayout->setObjectName("hboxLayout");
        frame_2 = new QFrame(grpFilters);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Raised);
        vboxLayout1 = new QVBoxLayout(frame_2);
        vboxLayout1->setSpacing(5);
        vboxLayout1->setContentsMargins(5, 5, 5, 5);
        vboxLayout1->setObjectName("vboxLayout1");
        label_10 = new QLabel(frame_2);
        label_10->setObjectName("label_10");

        vboxLayout1->addWidget(label_10);

        lvFilters = new QListWidget(frame_2);
        lvFilters->setObjectName("lvFilters");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lvFilters->sizePolicy().hasHeightForWidth());
        lvFilters->setSizePolicy(sizePolicy);

        vboxLayout1->addWidget(lvFilters);


        hboxLayout->addWidget(frame_2);

        frame = new QFrame(grpFilters);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout = new QVBoxLayout(frame);
        verticalLayout->setObjectName("verticalLayout");
        label_11 = new QLabel(frame);
        label_11->setObjectName("label_11");

        verticalLayout->addWidget(label_11);

        edFilterName = new QLineEdit(frame);
        edFilterName->setObjectName("edFilterName");
        edFilterName->setMinimumSize(QSize(100, 0));

        verticalLayout->addWidget(edFilterName);

        label_5 = new QLabel(frame);
        label_5->setObjectName("label_5");

        verticalLayout->addWidget(label_5);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        edFilterString = new QLineEdit(frame);
        edFilterString->setObjectName("edFilterString");

        horizontalLayout->addWidget(edFilterString);

        btFilterHelper = new QToolButton(frame);
        btFilterHelper->setObjectName("btFilterHelper");

        horizontalLayout->addWidget(btFilterHelper);


        verticalLayout->addLayout(horizontalLayout);

        spacerItem = new QSpacerItem(20, 1, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(spacerItem);

        btApply = new QPushButton(frame);
        btApply->setObjectName("btApply");

        verticalLayout->addWidget(btApply);

        btAdd = new QPushButton(frame);
        btAdd->setObjectName("btAdd");

        verticalLayout->addWidget(btAdd);

        btDel = new QPushButton(frame);
        btDel->setObjectName("btDel");

        verticalLayout->addWidget(btDel);


        hboxLayout->addWidget(frame);


        vboxLayout->addWidget(grpFilters);

        buttonBox = new QDialogButtonBox(FilterPreferencesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(FilterPreferencesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, FilterPreferencesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, FilterPreferencesDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(FilterPreferencesDialog);
    } // setupUi

    void retranslateUi(QDialog *FilterPreferencesDialog)
    {
        FilterPreferencesDialog->setWindowTitle(QCoreApplication::translate("FilterPreferencesDialog", "Filters setup", nullptr));
        grpFilters->setTitle(QCoreApplication::translate("FilterPreferencesDialog", "Filters", nullptr));
        label_10->setText(QCoreApplication::translate("FilterPreferencesDialog", "Filter list:", nullptr));
        label_11->setText(QCoreApplication::translate("FilterPreferencesDialog", "Name:", nullptr));
        label_5->setText(QCoreApplication::translate("FilterPreferencesDialog", "Filter", nullptr));
        btFilterHelper->setText(QCoreApplication::translate("FilterPreferencesDialog", "...", nullptr));
        btApply->setText(QCoreApplication::translate("FilterPreferencesDialog", "Apply", nullptr));
        btAdd->setText(QCoreApplication::translate("FilterPreferencesDialog", "Add", nullptr));
        btDel->setText(QCoreApplication::translate("FilterPreferencesDialog", "Remove", nullptr));
    } // retranslateUi

};

namespace Ui {
    class FilterPreferencesDialog: public Ui_FilterPreferencesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FILTERPREFERENCESDIALOG_H
