/********************************************************************************
** Form generated from reading UI file 'GotoDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GOTODIALOG_H
#define UI_GOTODIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_GotoDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_4;
    QRadioButton *rbBookmark;
    QComboBox *coordBookmark;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *rbLink;
    QLineEdit *coordLink;
    QGroupBox *groupBox_2;
    QVBoxLayout *verticalLayout_5;
    QCheckBox *cbCoordShowProjected;
    QHBoxLayout *horizontalLayout_3;
    QRadioButton *rbCoord;
    QLineEdit *coordCoord;
    QHBoxLayout *horizontalLayout;
    QRadioButton *rbSpan;
    QLineEdit *coordSpan;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label;
    QLineEdit *coordOsmApi;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label_2;
    QLineEdit *coordOsmXApi;
    QGroupBox *searchBox;
    QVBoxLayout *verticalLayout_3;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_7;
    QLineEdit *NameFinderEdit;
    QPushButton *searchButton;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *GotoDialog)
    {
        if (GotoDialog->objectName().isEmpty())
            GotoDialog->setObjectName("GotoDialog");
        GotoDialog->resize(600, 328);
        verticalLayout = new QVBoxLayout(GotoDialog);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        rbBookmark = new QRadioButton(GotoDialog);
        rbBookmark->setObjectName("rbBookmark");
        rbBookmark->setMinimumSize(QSize(200, 0));

        horizontalLayout_4->addWidget(rbBookmark);

        coordBookmark = new QComboBox(GotoDialog);
        coordBookmark->setObjectName("coordBookmark");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(coordBookmark->sizePolicy().hasHeightForWidth());
        coordBookmark->setSizePolicy(sizePolicy);
        coordBookmark->setMinimumSize(QSize(200, 0));

        horizontalLayout_4->addWidget(coordBookmark);


        verticalLayout->addLayout(horizontalLayout_4);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        rbLink = new QRadioButton(GotoDialog);
        rbLink->setObjectName("rbLink");
        rbLink->setMinimumSize(QSize(200, 0));

        horizontalLayout_2->addWidget(rbLink);

        coordLink = new QLineEdit(GotoDialog);
        coordLink->setObjectName("coordLink");
        sizePolicy.setHeightForWidth(coordLink->sizePolicy().hasHeightForWidth());
        coordLink->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(coordLink);


        verticalLayout->addLayout(horizontalLayout_2);

        groupBox_2 = new QGroupBox(GotoDialog);
        groupBox_2->setObjectName("groupBox_2");
        verticalLayout_5 = new QVBoxLayout(groupBox_2);
        verticalLayout_5->setSpacing(4);
        verticalLayout_5->setContentsMargins(4, 4, 4, 4);
        verticalLayout_5->setObjectName("verticalLayout_5");
        cbCoordShowProjected = new QCheckBox(groupBox_2);
        cbCoordShowProjected->setObjectName("cbCoordShowProjected");

        verticalLayout_5->addWidget(cbCoordShowProjected);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        rbCoord = new QRadioButton(groupBox_2);
        rbCoord->setObjectName("rbCoord");
        rbCoord->setMinimumSize(QSize(200, 0));

        horizontalLayout_3->addWidget(rbCoord);

        coordCoord = new QLineEdit(groupBox_2);
        coordCoord->setObjectName("coordCoord");
        sizePolicy.setHeightForWidth(coordCoord->sizePolicy().hasHeightForWidth());
        coordCoord->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(coordCoord);


        verticalLayout_5->addLayout(horizontalLayout_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        rbSpan = new QRadioButton(groupBox_2);
        rbSpan->setObjectName("rbSpan");
        rbSpan->setMinimumSize(QSize(200, 0));

        horizontalLayout->addWidget(rbSpan);

        coordSpan = new QLineEdit(groupBox_2);
        coordSpan->setObjectName("coordSpan");
        sizePolicy.setHeightForWidth(coordSpan->sizePolicy().hasHeightForWidth());
        coordSpan->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(coordSpan);


        verticalLayout_5->addLayout(horizontalLayout);


        verticalLayout->addWidget(groupBox_2);

        groupBox = new QGroupBox(GotoDialog);
        groupBox->setObjectName("groupBox");
        verticalLayout_2 = new QVBoxLayout(groupBox);
        verticalLayout_2->setSpacing(4);
        verticalLayout_2->setContentsMargins(4, 4, 4, 4);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label = new QLabel(groupBox);
        label->setObjectName("label");
        label->setMinimumSize(QSize(75, 0));

        horizontalLayout_5->addWidget(label);

        coordOsmApi = new QLineEdit(groupBox);
        coordOsmApi->setObjectName("coordOsmApi");
        coordOsmApi->setEnabled(true);
        coordOsmApi->setReadOnly(true);

        horizontalLayout_5->addWidget(coordOsmApi);


        verticalLayout_2->addLayout(horizontalLayout_5);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        label_2 = new QLabel(groupBox);
        label_2->setObjectName("label_2");
        label_2->setMinimumSize(QSize(75, 0));

        horizontalLayout_6->addWidget(label_2);

        coordOsmXApi = new QLineEdit(groupBox);
        coordOsmXApi->setObjectName("coordOsmXApi");
        coordOsmXApi->setEnabled(true);
        coordOsmXApi->setReadOnly(true);

        horizontalLayout_6->addWidget(coordOsmXApi);


        verticalLayout_2->addLayout(horizontalLayout_6);


        verticalLayout->addWidget(groupBox);

        searchBox = new QGroupBox(GotoDialog);
        searchBox->setObjectName("searchBox");
        verticalLayout_3 = new QVBoxLayout(searchBox);
        verticalLayout_3->setSpacing(4);
        verticalLayout_3->setContentsMargins(4, 4, 4, 4);
        verticalLayout_3->setObjectName("verticalLayout_3");
        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName("verticalLayout_4");
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        NameFinderEdit = new QLineEdit(searchBox);
        NameFinderEdit->setObjectName("NameFinderEdit");

        horizontalLayout_7->addWidget(NameFinderEdit);

        searchButton = new QPushButton(searchBox);
        searchButton->setObjectName("searchButton");

        horizontalLayout_7->addWidget(searchButton);


        verticalLayout_4->addLayout(horizontalLayout_7);


        verticalLayout_3->addLayout(verticalLayout_4);


        verticalLayout->addWidget(searchBox);

        buttonBox = new QDialogButtonBox(GotoDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

        QWidget::setTabOrder(coordBookmark, coordLink);
        QWidget::setTabOrder(coordLink, coordCoord);
        QWidget::setTabOrder(coordCoord, coordSpan);
        QWidget::setTabOrder(coordSpan, coordOsmApi);
        QWidget::setTabOrder(coordOsmApi, coordOsmXApi);
        QWidget::setTabOrder(coordOsmXApi, NameFinderEdit);
        QWidget::setTabOrder(NameFinderEdit, searchButton);
        QWidget::setTabOrder(searchButton, buttonBox);
        QWidget::setTabOrder(buttonBox, rbBookmark);
        QWidget::setTabOrder(rbBookmark, rbLink);
        QWidget::setTabOrder(rbLink, rbCoord);
        QWidget::setTabOrder(rbCoord, rbSpan);

        retranslateUi(GotoDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, GotoDialog, qOverload<>(&QDialog::reject));
        QObject::connect(coordSpan, &QLineEdit::textEdited, rbSpan, qOverload<>(&QRadioButton::toggle));
        QObject::connect(coordCoord, &QLineEdit::textEdited, rbCoord, qOverload<>(&QRadioButton::toggle));
        QObject::connect(coordLink, &QLineEdit::textEdited, rbLink, qOverload<>(&QRadioButton::toggle));
        QObject::connect(coordBookmark, &QComboBox::activated, rbBookmark, qOverload<>(&QRadioButton::toggle));
        QObject::connect(coordBookmark, &QComboBox::highlighted, rbBookmark, qOverload<>(&QRadioButton::toggle));

        QMetaObject::connectSlotsByName(GotoDialog);
    } // setupUi

    void retranslateUi(QDialog *GotoDialog)
    {
        GotoDialog->setWindowTitle(QCoreApplication::translate("GotoDialog", "Go To", nullptr));
        rbBookmark->setText(QCoreApplication::translate("GotoDialog", "Bookmark", nullptr));
        rbLink->setText(QCoreApplication::translate("GotoDialog", "WWW link (OSM/Google maps)", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("GotoDialog", "Coordinates", nullptr));
        cbCoordShowProjected->setText(QCoreApplication::translate("GotoDialog", "Show projected", nullptr));
        rbCoord->setText(QCoreApplication::translate("GotoDialog", "lonMin, latMin, lonMax, latMax", nullptr));
        rbSpan->setText(QCoreApplication::translate("GotoDialog", "latCent, lonCent, latSpan, lonSpan", nullptr));
        groupBox->setTitle(QCoreApplication::translate("GotoDialog", "Info", nullptr));
        label->setText(QCoreApplication::translate("GotoDialog", "OSM API Url", nullptr));
        label_2->setText(QCoreApplication::translate("GotoDialog", "OSM XAPI url", nullptr));
        searchBox->setTitle(QCoreApplication::translate("GotoDialog", "Search", nullptr));
        searchButton->setText(QCoreApplication::translate("GotoDialog", "Search", nullptr));
    } // retranslateUi

};

namespace Ui {
    class GotoDialog: public Ui_GotoDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GOTODIALOG_H
