/********************************************************************************
** Form generated from reading UI file 'NativeRenderDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NATIVERENDERDIALOG_H
#define UI_NATIVERENDERDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_NativeRenderWidget
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *grpOptions;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QDoubleSpinBox *sbMinLat;
    QLabel *label_4;
    QDoubleSpinBox *sbMaxLat;
    QDoubleSpinBox *sbMinLon;
    QDoubleSpinBox *sbMaxLon;
    QFrame *line;
    QGridLayout *gridLayout_2;
    QCheckBox *cbShowNodes;
    QCheckBox *cbShowRelations;
    QCheckBox *cbShowScale;
    QCheckBox *cbShowGrid;
    QCheckBox *cbShowUnstyled;
    QFrame *line_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *labelPrintResolution;
    QComboBox *fieldDpi;
    QLabel *labelDpi;
    QSpacerItem *horizontalSpacer;
    QFrame *line_3;
    QHBoxLayout *horizontalLayout;
    QPushButton *btExportPDF;
    QPushButton *btExportSVG;
    QPushButton *btExportRaster;

    void setupUi(QWidget *NativeRenderWidget)
    {
        if (NativeRenderWidget->objectName().isEmpty())
            NativeRenderWidget->setObjectName("NativeRenderWidget");
        NativeRenderWidget->resize(714, 629);
        verticalLayout = new QVBoxLayout(NativeRenderWidget);
        verticalLayout->setSpacing(4);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(4, 4, 4, 4);
        grpOptions = new QGroupBox(NativeRenderWidget);
        grpOptions->setObjectName("grpOptions");
        verticalLayout_3 = new QVBoxLayout(grpOptions);
        verticalLayout_3->setObjectName("verticalLayout_3");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(4);
        gridLayout->setObjectName("gridLayout");
        label_3 = new QLabel(grpOptions);
        label_3->setObjectName("label_3");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy);
        label_3->setMinimumSize(QSize(60, 0));

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        sbMinLat = new QDoubleSpinBox(grpOptions);
        sbMinLat->setObjectName("sbMinLat");
        sbMinLat->setMinimumSize(QSize(60, 0));
        sbMinLat->setDecimals(6);
        sbMinLat->setMinimum(-90.000000000000000);
        sbMinLat->setMaximum(90.000000000000000);
        sbMinLat->setSingleStep(0.010000000000000);

        gridLayout->addWidget(sbMinLat, 0, 1, 1, 1);

        label_4 = new QLabel(grpOptions);
        label_4->setObjectName("label_4");
        sizePolicy.setHeightForWidth(label_4->sizePolicy().hasHeightForWidth());
        label_4->setSizePolicy(sizePolicy);
        label_4->setMinimumSize(QSize(60, 0));

        gridLayout->addWidget(label_4, 2, 0, 1, 1);

        sbMaxLat = new QDoubleSpinBox(grpOptions);
        sbMaxLat->setObjectName("sbMaxLat");
        sbMaxLat->setMinimumSize(QSize(60, 0));
        sbMaxLat->setDecimals(6);
        sbMaxLat->setMinimum(-90.000000000000000);
        sbMaxLat->setMaximum(90.000000000000000);
        sbMaxLat->setSingleStep(0.010000000000000);

        gridLayout->addWidget(sbMaxLat, 2, 1, 1, 1);

        sbMinLon = new QDoubleSpinBox(grpOptions);
        sbMinLon->setObjectName("sbMinLon");
        sbMinLon->setMinimumSize(QSize(60, 0));
        sbMinLon->setDecimals(6);
        sbMinLon->setMinimum(-180.000000000000000);
        sbMinLon->setMaximum(180.000000000000000);
        sbMinLon->setSingleStep(0.010000000000000);

        gridLayout->addWidget(sbMinLon, 0, 2, 1, 1);

        sbMaxLon = new QDoubleSpinBox(grpOptions);
        sbMaxLon->setObjectName("sbMaxLon");
        sbMaxLon->setMinimumSize(QSize(60, 0));
        sbMaxLon->setDecimals(6);
        sbMaxLon->setMinimum(-180.000000000000000);
        sbMaxLon->setMaximum(180.000000000000000);
        sbMaxLon->setSingleStep(0.010000000000000);

        gridLayout->addWidget(sbMaxLon, 2, 2, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        line = new QFrame(grpOptions);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        cbShowNodes = new QCheckBox(grpOptions);
        cbShowNodes->setObjectName("cbShowNodes");

        gridLayout_2->addWidget(cbShowNodes, 0, 0, 1, 1);

        cbShowRelations = new QCheckBox(grpOptions);
        cbShowRelations->setObjectName("cbShowRelations");

        gridLayout_2->addWidget(cbShowRelations, 0, 1, 1, 1);

        cbShowScale = new QCheckBox(grpOptions);
        cbShowScale->setObjectName("cbShowScale");
        cbShowScale->setEnabled(true);
        cbShowScale->setTristate(false);

        gridLayout_2->addWidget(cbShowScale, 0, 2, 1, 1);

        cbShowGrid = new QCheckBox(grpOptions);
        cbShowGrid->setObjectName("cbShowGrid");
        cbShowGrid->setEnabled(true);
        cbShowGrid->setTristate(false);

        gridLayout_2->addWidget(cbShowGrid, 0, 3, 1, 1);

        cbShowUnstyled = new QCheckBox(grpOptions);
        cbShowUnstyled->setObjectName("cbShowUnstyled");

        gridLayout_2->addWidget(cbShowUnstyled, 1, 0, 1, 1);


        verticalLayout_3->addLayout(gridLayout_2);

        line_2 = new QFrame(grpOptions);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line_2);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        labelPrintResolution = new QLabel(grpOptions);
        labelPrintResolution->setObjectName("labelPrintResolution");

        horizontalLayout_2->addWidget(labelPrintResolution);

        fieldDpi = new QComboBox(grpOptions);
        fieldDpi->addItem(QString());
        fieldDpi->addItem(QString());
        fieldDpi->addItem(QString());
        fieldDpi->setObjectName("fieldDpi");
        fieldDpi->setEditable(true);
        fieldDpi->setCurrentText(QString::fromUtf8("300"));

        horizontalLayout_2->addWidget(fieldDpi);

        labelDpi = new QLabel(grpOptions);
        labelDpi->setObjectName("labelDpi");

        horizontalLayout_2->addWidget(labelDpi);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout_2);

        line_3 = new QFrame(grpOptions);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_3->addWidget(line_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btExportPDF = new QPushButton(grpOptions);
        btExportPDF->setObjectName("btExportPDF");

        horizontalLayout->addWidget(btExportPDF);

        btExportSVG = new QPushButton(grpOptions);
        btExportSVG->setObjectName("btExportSVG");

        horizontalLayout->addWidget(btExportSVG);

        btExportRaster = new QPushButton(grpOptions);
        btExportRaster->setObjectName("btExportRaster");

        horizontalLayout->addWidget(btExportRaster);


        verticalLayout_3->addLayout(horizontalLayout);


        verticalLayout->addWidget(grpOptions);


        retranslateUi(NativeRenderWidget);

        QMetaObject::connectSlotsByName(NativeRenderWidget);
    } // setupUi

    void retranslateUi(QWidget *NativeRenderWidget)
    {
        NativeRenderWidget->setWindowTitle(QCoreApplication::translate("NativeRenderWidget", "Raster/SVG export", nullptr));
        grpOptions->setTitle(QCoreApplication::translate("NativeRenderWidget", "Options", nullptr));
        label_3->setText(QCoreApplication::translate("NativeRenderWidget", "min lat/Lon", nullptr));
        label_4->setText(QCoreApplication::translate("NativeRenderWidget", "max lat/Lon", nullptr));
        cbShowNodes->setText(QCoreApplication::translate("NativeRenderWidget", "Show Nodes", nullptr));
        cbShowRelations->setText(QCoreApplication::translate("NativeRenderWidget", "Show Relations", nullptr));
        cbShowScale->setText(QCoreApplication::translate("NativeRenderWidget", "Show Scale", nullptr));
        cbShowGrid->setText(QCoreApplication::translate("NativeRenderWidget", "Show Lat/Lon Grid", nullptr));
        cbShowUnstyled->setText(QCoreApplication::translate("NativeRenderWidget", "Show unstyled features", nullptr));
        labelPrintResolution->setText(QCoreApplication::translate("NativeRenderWidget", "Print resolution", nullptr));
        fieldDpi->setItemText(0, QCoreApplication::translate("NativeRenderWidget", "300", nullptr));
        fieldDpi->setItemText(1, QCoreApplication::translate("NativeRenderWidget", "600", nullptr));
        fieldDpi->setItemText(2, QCoreApplication::translate("NativeRenderWidget", "1200", nullptr));

        labelDpi->setText(QCoreApplication::translate("NativeRenderWidget", "dpi", nullptr));
        btExportPDF->setText(QCoreApplication::translate("NativeRenderWidget", "Export to PDF...", nullptr));
        btExportSVG->setText(QCoreApplication::translate("NativeRenderWidget", "Export to SVG...", nullptr));
        btExportRaster->setText(QCoreApplication::translate("NativeRenderWidget", "Export to Raster...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class NativeRenderWidget: public Ui_NativeRenderWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NATIVERENDERDIALOG_H
