/********************************************************************************
** Form generated from reading UI file 'MinimumRelationProperties.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MINIMUMRELATIONPROPERTIES_H
#define UI_MINIMUMRELATIONPROPERTIES_H

#include <QtCore/QVariant>
#include <QtGui/QIcon>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_RelationProperties
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *variableLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *MembersLabel;
    QSpacerItem *horizontalSpacer;
    QToolButton *RemoveMemberButton;
    QHBoxLayout *horizontalLayout_2;
    QTableView *MembersView;
    QVBoxLayout *verticalLayout;
    QToolButton *btMemberUp;
    QToolButton *btMemberDown;
    QHBoxLayout *hboxLayout;
    QLabel *label_4;
    QSpacerItem *spacerItem;
    QToolButton *SourceTagButton;
    QToolButton *RemoveTagButton;
    QTableView *TagView;

    void setupUi(QWidget *RelationProperties)
    {
        if (RelationProperties->objectName().isEmpty())
            RelationProperties->setObjectName("RelationProperties");
        RelationProperties->resize(228, 495);
        vboxLayout = new QVBoxLayout(RelationProperties);
        vboxLayout->setSpacing(4);
        vboxLayout->setContentsMargins(4, 4, 4, 4);
        vboxLayout->setObjectName("vboxLayout");
        variableLayout = new QGridLayout();
        variableLayout->setObjectName("variableLayout");

        vboxLayout->addLayout(variableLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        MembersLabel = new QLabel(RelationProperties);
        MembersLabel->setObjectName("MembersLabel");

        horizontalLayout->addWidget(MembersLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        RemoveMemberButton = new QToolButton(RelationProperties);
        RemoveMemberButton->setObjectName("RemoveMemberButton");
        QIcon icon;
        icon.addFile(QString::fromUtf8(":/Icons/small-remove.xpm"), QSize(), QIcon::Mode::Normal, QIcon::State::Off);
        RemoveMemberButton->setIcon(icon);

        horizontalLayout->addWidget(RemoveMemberButton);


        vboxLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        MembersView = new QTableView(RelationProperties);
        MembersView->setObjectName("MembersView");
        MembersView->setTabKeyNavigation(false);

        horizontalLayout_2->addWidget(MembersView);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName("verticalLayout");
        btMemberUp = new QToolButton(RelationProperties);
        btMemberUp->setObjectName("btMemberUp");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(btMemberUp->sizePolicy().hasHeightForWidth());
        btMemberUp->setSizePolicy(sizePolicy);
        btMemberUp->setArrowType(Qt::UpArrow);

        verticalLayout->addWidget(btMemberUp);

        btMemberDown = new QToolButton(RelationProperties);
        btMemberDown->setObjectName("btMemberDown");
        sizePolicy.setHeightForWidth(btMemberDown->sizePolicy().hasHeightForWidth());
        btMemberDown->setSizePolicy(sizePolicy);
        btMemberDown->setArrowType(Qt::DownArrow);

        verticalLayout->addWidget(btMemberDown);


        horizontalLayout_2->addLayout(verticalLayout);


        vboxLayout->addLayout(horizontalLayout_2);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(4);
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        label_4 = new QLabel(RelationProperties);
        label_4->setObjectName("label_4");

        hboxLayout->addWidget(label_4);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        SourceTagButton = new QToolButton(RelationProperties);
        SourceTagButton->setObjectName("SourceTagButton");
        SourceTagButton->setMinimumSize(QSize(23, 23));

        hboxLayout->addWidget(SourceTagButton);

        RemoveTagButton = new QToolButton(RelationProperties);
        RemoveTagButton->setObjectName("RemoveTagButton");
        RemoveTagButton->setMinimumSize(QSize(23, 23));
        RemoveTagButton->setIcon(icon);

        hboxLayout->addWidget(RemoveTagButton);


        vboxLayout->addLayout(hboxLayout);

        TagView = new QTableView(RelationProperties);
        TagView->setObjectName("TagView");
        TagView->setTabKeyNavigation(false);

        vboxLayout->addWidget(TagView);


        retranslateUi(RelationProperties);

        QMetaObject::connectSlotsByName(RelationProperties);
    } // setupUi

    void retranslateUi(QWidget *RelationProperties)
    {
        RelationProperties->setWindowTitle(QCoreApplication::translate("RelationProperties", "Form", nullptr));
        MembersLabel->setText(QCoreApplication::translate("RelationProperties", "Members", nullptr));
#if QT_CONFIG(tooltip)
        RemoveMemberButton->setToolTip(QCoreApplication::translate("RelationProperties", "Remove selected member(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        RemoveMemberButton->setText(QCoreApplication::translate("RelationProperties", "...", nullptr));
        btMemberUp->setText(QCoreApplication::translate("RelationProperties", "...", nullptr));
        btMemberDown->setText(QCoreApplication::translate("RelationProperties", "...", nullptr));
        label_4->setText(QCoreApplication::translate("RelationProperties", "Tags", nullptr));
#if QT_CONFIG(tooltip)
        SourceTagButton->setToolTip(QCoreApplication::translate("RelationProperties", "Add \"source\" tag", nullptr));
#endif // QT_CONFIG(tooltip)
        SourceTagButton->setText(QCoreApplication::translate("RelationProperties", "S", nullptr));
#if QT_CONFIG(tooltip)
        RemoveTagButton->setToolTip(QCoreApplication::translate("RelationProperties", "Remove selected tag(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        RemoveTagButton->setText(QCoreApplication::translate("RelationProperties", "...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RelationProperties: public Ui_RelationProperties {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MINIMUMRELATIONPROPERTIES_H
