/********************************************************************************
** Form generated from reading UI file 'AboutDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTDIALOG_H
#define UI_ABOUTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_AboutDialog
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QGridLayout *gridLayout;
    QLabel *ProjVersion;
    QLabel *Version;
    QLabel *VersionLabel;
    QLabel *QTVersion;
    QLabel *GdalVersion;
    QLabel *QTVersionLabel;
    QLabel *ProjVersionLabel;
    QLabel *GdalVersionLabel;
    QSpacerItem *horizontalSpacer;
    QLabel *pxIcon;
    QLabel *label;
    QLabel *label_3;
    QLabel *lblUrl;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QTextEdit *txtChangelog;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *okButton;

    void setupUi(QDialog *AboutDialog)
    {
        if (AboutDialog->objectName().isEmpty())
            AboutDialog->setObjectName("AboutDialog");
        AboutDialog->resize(525, 469);
        AboutDialog->setSizeGripEnabled(true);
        verticalLayout_3 = new QVBoxLayout(AboutDialog);
        verticalLayout_3->setObjectName("verticalLayout_3");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        gridLayout->setHorizontalSpacing(20);
        gridLayout->setVerticalSpacing(-1);
        ProjVersion = new QLabel(AboutDialog);
        ProjVersion->setObjectName("ProjVersion");
        ProjVersion->setStyleSheet(QString::fromUtf8("QLabel { font-size:small; font-weight:bold; }"));
        ProjVersion->setText(QString::fromUtf8("%1"));

        gridLayout->addWidget(ProjVersion, 2, 1, 1, 1);

        Version = new QLabel(AboutDialog);
        Version->setObjectName("Version");
        Version->setStyleSheet(QString::fromUtf8("QLabel { font-size:small; font-weight:bold; }"));
        Version->setText(QString::fromUtf8("%1"));

        gridLayout->addWidget(Version, 0, 1, 1, 1);

        VersionLabel = new QLabel(AboutDialog);
        VersionLabel->setObjectName("VersionLabel");
        VersionLabel->setStyleSheet(QString::fromUtf8("QLabel { font-size: x-small; font-style: italic; }"));

        gridLayout->addWidget(VersionLabel, 0, 0, 1, 1);

        QTVersion = new QLabel(AboutDialog);
        QTVersion->setObjectName("QTVersion");
        QTVersion->setStyleSheet(QString::fromUtf8("QLabel { font-size:small; font-weight:bold; }"));
        QTVersion->setText(QString::fromUtf8("%1 (%2)"));

        gridLayout->addWidget(QTVersion, 1, 1, 1, 1);

        GdalVersion = new QLabel(AboutDialog);
        GdalVersion->setObjectName("GdalVersion");
        GdalVersion->setStyleSheet(QString::fromUtf8("QLabel { font-size:small; font-weight:bold; }"));
        GdalVersion->setText(QString::fromUtf8("%1"));

        gridLayout->addWidget(GdalVersion, 3, 1, 1, 1);

        QTVersionLabel = new QLabel(AboutDialog);
        QTVersionLabel->setObjectName("QTVersionLabel");
        QTVersionLabel->setStyleSheet(QString::fromUtf8("QLabel { font-size: x-small; font-style: italic; }"));

        gridLayout->addWidget(QTVersionLabel, 1, 0, 1, 1);

        ProjVersionLabel = new QLabel(AboutDialog);
        ProjVersionLabel->setObjectName("ProjVersionLabel");
        ProjVersionLabel->setStyleSheet(QString::fromUtf8("QLabel { font-size: x-small; font-style: italic; }"));

        gridLayout->addWidget(ProjVersionLabel, 2, 0, 1, 1);

        GdalVersionLabel = new QLabel(AboutDialog);
        GdalVersionLabel->setObjectName("GdalVersionLabel");
        GdalVersionLabel->setStyleSheet(QString::fromUtf8("QLabel { font-size: x-small; font-style: italic; }"));

        gridLayout->addWidget(GdalVersionLabel, 3, 0, 1, 1);


        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        pxIcon = new QLabel(AboutDialog);
        pxIcon->setObjectName("pxIcon");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(100);
        sizePolicy.setVerticalStretch(100);
        sizePolicy.setHeightForWidth(pxIcon->sizePolicy().hasHeightForWidth());
        pxIcon->setSizePolicy(sizePolicy);
        pxIcon->setMinimumSize(QSize(100, 100));
        pxIcon->setText(QString::fromUtf8("TextLabel"));

        horizontalLayout->addWidget(pxIcon);


        verticalLayout_3->addLayout(horizontalLayout);

        label = new QLabel(AboutDialog);
        label->setObjectName("label");

        verticalLayout_3->addWidget(label);

        label_3 = new QLabel(AboutDialog);
        label_3->setObjectName("label_3");

        verticalLayout_3->addWidget(label_3);

        lblUrl = new QLabel(AboutDialog);
        lblUrl->setObjectName("lblUrl");
        lblUrl->setText(QString::fromUtf8("<a href=\"http://merkaartor.be/\">http://merkaartor.be/</a>"));
        lblUrl->setOpenExternalLinks(true);
        lblUrl->setTextInteractionFlags(Qt::LinksAccessibleByMouse|Qt::TextSelectableByMouse);

        verticalLayout_3->addWidget(lblUrl);

        groupBox = new QGroupBox(AboutDialog);
        groupBox->setObjectName("groupBox");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy1);
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName("verticalLayout");
        txtChangelog = new QTextEdit(groupBox);
        txtChangelog->setObjectName("txtChangelog");
        txtChangelog->setLineWrapMode(QTextEdit::NoWrap);
        txtChangelog->setReadOnly(true);
        txtChangelog->setTabStopDistance(4);

        verticalLayout->addWidget(txtChangelog);


        verticalLayout_3->addWidget(groupBox);

        hboxLayout = new QHBoxLayout();
#ifndef Q_OS_MAC
        hboxLayout->setSpacing(6);
#endif
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        hboxLayout->setObjectName("hboxLayout");
        spacerItem = new QSpacerItem(131, 31, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);

        okButton = new QPushButton(AboutDialog);
        okButton->setObjectName("okButton");

        hboxLayout->addWidget(okButton);


        verticalLayout_3->addLayout(hboxLayout);


        retranslateUi(AboutDialog);
        QObject::connect(okButton, &QPushButton::clicked, AboutDialog, qOverload<>(&QDialog::accept));

        QMetaObject::connectSlotsByName(AboutDialog);
    } // setupUi

    void retranslateUi(QDialog *AboutDialog)
    {
        AboutDialog->setWindowTitle(QCoreApplication::translate("AboutDialog", "About Merkaartor", nullptr));
        VersionLabel->setText(QCoreApplication::translate("AboutDialog", "Merkaartor version", nullptr));
        QTVersionLabel->setText(QCoreApplication::translate("AboutDialog", "Qt version", nullptr));
        ProjVersionLabel->setText(QCoreApplication::translate("AboutDialog", "PROJ version", nullptr));
        GdalVersionLabel->setText(QCoreApplication::translate("AboutDialog", "GDAL version", nullptr));
        label->setText(QCoreApplication::translate("AboutDialog", "Copyright Bart Vanhauwaert, Chris Browet, Ladislav L\303\241ska and others, 2006-2021", nullptr));
        label_3->setText(QCoreApplication::translate("AboutDialog", "This program is licensed under the GNU General Public License v2", nullptr));
        groupBox->setTitle(QCoreApplication::translate("AboutDialog", "Changelog", nullptr));
        okButton->setText(QCoreApplication::translate("AboutDialog", "OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutDialog: public Ui_AboutDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTDIALOG_H
