/********************************************************************************
** Form generated from reading UI file 'WMSPreferencesDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WMSPREFERENCESDIALOG_H
#define UI_WMSPREFERENCESDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_WMSPreferencesDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *grpWmsServers;
    QHBoxLayout *hboxLayout;
    QFrame *frame_2;
    QVBoxLayout *vboxLayout1;
    QLabel *label_10;
    QListWidget *lvWmsServers;
    QFrame *frame;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_11;
    QLineEdit *edWmsName;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_5;
    QLineEdit *edWmsUrl;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_7;
    QLineEdit *edWmsLayers;
    QFrame *line;
    QTreeWidget *tvWmsLayers;
    QLabel *lblStatus;
    QFrame *frWmsSettings;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label_8;
    QComboBox *cbWmsProj;
    QWidget *frTileIt;
    QHBoxLayout *horizontalLayout_2;
    QCheckBox *cbTileIt;
    QSpacerItem *horizontalSpacer;
    QLabel *label_2;
    QSpinBox *sbZoomLevels;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label;
    QComboBox *cbWmsImgFormat;
    QHBoxLayout *horizontalLayout_8;
    QLabel *label_9;
    QComboBox *cbWmsStyle;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_3;
    QLineEdit *edSourceTag;
    QHBoxLayout *horizontalLayout_10;
    QLabel *label_4;
    QLineEdit *edLicenseUrl;
    QHBoxLayout *horizontalLayout;
    QPushButton *btApplyWmsServer;
    QPushButton *btAddWmsServer;
    QPushButton *btDelWmsServer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *WMSPreferencesDialog)
    {
        if (WMSPreferencesDialog->objectName().isEmpty())
            WMSPreferencesDialog->setObjectName("WMSPreferencesDialog");
        WMSPreferencesDialog->resize(797, 718);
        vboxLayout = new QVBoxLayout(WMSPreferencesDialog);
        vboxLayout->setObjectName("vboxLayout");
        grpWmsServers = new QGroupBox(WMSPreferencesDialog);
        grpWmsServers->setObjectName("grpWmsServers");
        hboxLayout = new QHBoxLayout(grpWmsServers);
        hboxLayout->setSpacing(1);
        hboxLayout->setContentsMargins(1, 1, 1, 1);
        hboxLayout->setObjectName("hboxLayout");
        frame_2 = new QFrame(grpWmsServers);
        frame_2->setObjectName("frame_2");
        frame_2->setFrameShape(QFrame::StyledPanel);
        frame_2->setFrameShadow(QFrame::Raised);
        vboxLayout1 = new QVBoxLayout(frame_2);
        vboxLayout1->setSpacing(5);
        vboxLayout1->setContentsMargins(5, 5, 5, 5);
        vboxLayout1->setObjectName("vboxLayout1");
        label_10 = new QLabel(frame_2);
        label_10->setObjectName("label_10");

        vboxLayout1->addWidget(label_10);

        lvWmsServers = new QListWidget(frame_2);
        lvWmsServers->setObjectName("lvWmsServers");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(lvWmsServers->sizePolicy().hasHeightForWidth());
        lvWmsServers->setSizePolicy(sizePolicy);

        vboxLayout1->addWidget(lvWmsServers);


        hboxLayout->addWidget(frame_2);

        frame = new QFrame(grpWmsServers);
        frame->setObjectName("frame");
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout_2 = new QVBoxLayout(frame);
        verticalLayout_2->setObjectName("verticalLayout_2");
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName("horizontalLayout_3");
        label_11 = new QLabel(frame);
        label_11->setObjectName("label_11");

        horizontalLayout_3->addWidget(label_11);

        edWmsName = new QLineEdit(frame);
        edWmsName->setObjectName("edWmsName");
        edWmsName->setMinimumSize(QSize(100, 0));

        horizontalLayout_3->addWidget(edWmsName);


        verticalLayout_2->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName("horizontalLayout_4");
        label_5 = new QLabel(frame);
        label_5->setObjectName("label_5");

        horizontalLayout_4->addWidget(label_5);

        edWmsUrl = new QLineEdit(frame);
        edWmsUrl->setObjectName("edWmsUrl");

        horizontalLayout_4->addWidget(edWmsUrl);


        verticalLayout_2->addLayout(horizontalLayout_4);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName("horizontalLayout_5");
        label_7 = new QLabel(frame);
        label_7->setObjectName("label_7");

        horizontalLayout_5->addWidget(label_7);

        edWmsLayers = new QLineEdit(frame);
        edWmsLayers->setObjectName("edWmsLayers");

        horizontalLayout_5->addWidget(edWmsLayers);


        verticalLayout_2->addLayout(horizontalLayout_5);

        line = new QFrame(frame);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        verticalLayout_2->addWidget(line);

        tvWmsLayers = new QTreeWidget(frame);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(1, QString::fromUtf8("Name"));
        tvWmsLayers->setHeaderItem(__qtreewidgetitem);
        tvWmsLayers->setObjectName("tvWmsLayers");
        tvWmsLayers->setHeaderHidden(false);
        tvWmsLayers->setColumnCount(2);

        verticalLayout_2->addWidget(tvWmsLayers);

        lblStatus = new QLabel(frame);
        lblStatus->setObjectName("lblStatus");
        lblStatus->setText(QString::fromUtf8(""));
        lblStatus->setOpenExternalLinks(true);

        verticalLayout_2->addWidget(lblStatus);

        frWmsSettings = new QFrame(frame);
        frWmsSettings->setObjectName("frWmsSettings");
        verticalLayout = new QVBoxLayout(frWmsSettings);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName("horizontalLayout_6");
        label_8 = new QLabel(frWmsSettings);
        label_8->setObjectName("label_8");

        horizontalLayout_6->addWidget(label_8);

        cbWmsProj = new QComboBox(frWmsSettings);
        cbWmsProj->setObjectName("cbWmsProj");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(cbWmsProj->sizePolicy().hasHeightForWidth());
        cbWmsProj->setSizePolicy(sizePolicy1);
        cbWmsProj->setEditable(true);

        horizontalLayout_6->addWidget(cbWmsProj);


        verticalLayout->addLayout(horizontalLayout_6);

        frTileIt = new QWidget(frWmsSettings);
        frTileIt->setObjectName("frTileIt");
        horizontalLayout_2 = new QHBoxLayout(frTileIt);
        horizontalLayout_2->setSpacing(4);
        horizontalLayout_2->setContentsMargins(4, 4, 4, 4);
        horizontalLayout_2->setObjectName("horizontalLayout_2");
        cbTileIt = new QCheckBox(frTileIt);
        cbTileIt->setObjectName("cbTileIt");

        horizontalLayout_2->addWidget(cbTileIt);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        label_2 = new QLabel(frTileIt);
        label_2->setObjectName("label_2");

        horizontalLayout_2->addWidget(label_2);

        sbZoomLevels = new QSpinBox(frTileIt);
        sbZoomLevels->setObjectName("sbZoomLevels");
        sbZoomLevels->setMaximum(99);
        sbZoomLevels->setValue(17);

        horizontalLayout_2->addWidget(sbZoomLevels);


        verticalLayout->addWidget(frTileIt);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName("horizontalLayout_7");
        label = new QLabel(frWmsSettings);
        label->setObjectName("label");

        horizontalLayout_7->addWidget(label);

        cbWmsImgFormat = new QComboBox(frWmsSettings);
        cbWmsImgFormat->setObjectName("cbWmsImgFormat");
        sizePolicy1.setHeightForWidth(cbWmsImgFormat->sizePolicy().hasHeightForWidth());
        cbWmsImgFormat->setSizePolicy(sizePolicy1);
        cbWmsImgFormat->setEditable(true);

        horizontalLayout_7->addWidget(cbWmsImgFormat);


        verticalLayout->addLayout(horizontalLayout_7);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName("horizontalLayout_8");
        label_9 = new QLabel(frWmsSettings);
        label_9->setObjectName("label_9");

        horizontalLayout_8->addWidget(label_9);

        cbWmsStyle = new QComboBox(frWmsSettings);
        cbWmsStyle->setObjectName("cbWmsStyle");
        sizePolicy1.setHeightForWidth(cbWmsStyle->sizePolicy().hasHeightForWidth());
        cbWmsStyle->setSizePolicy(sizePolicy1);
        cbWmsStyle->setEditable(true);
        cbWmsStyle->setInsertPolicy(QComboBox::NoInsert);

        horizontalLayout_8->addWidget(cbWmsStyle);


        verticalLayout->addLayout(horizontalLayout_8);


        verticalLayout_2->addWidget(frWmsSettings);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName("horizontalLayout_9");
        label_3 = new QLabel(frame);
        label_3->setObjectName("label_3");

        horizontalLayout_9->addWidget(label_3);

        edSourceTag = new QLineEdit(frame);
        edSourceTag->setObjectName("edSourceTag");

        horizontalLayout_9->addWidget(edSourceTag);


        verticalLayout_2->addLayout(horizontalLayout_9);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName("horizontalLayout_10");
        label_4 = new QLabel(frame);
        label_4->setObjectName("label_4");

        horizontalLayout_10->addWidget(label_4);

        edLicenseUrl = new QLineEdit(frame);
        edLicenseUrl->setObjectName("edLicenseUrl");

        horizontalLayout_10->addWidget(edLicenseUrl);


        verticalLayout_2->addLayout(horizontalLayout_10);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        btApplyWmsServer = new QPushButton(frame);
        btApplyWmsServer->setObjectName("btApplyWmsServer");

        horizontalLayout->addWidget(btApplyWmsServer);

        btAddWmsServer = new QPushButton(frame);
        btAddWmsServer->setObjectName("btAddWmsServer");

        horizontalLayout->addWidget(btAddWmsServer);

        btDelWmsServer = new QPushButton(frame);
        btDelWmsServer->setObjectName("btDelWmsServer");

        horizontalLayout->addWidget(btDelWmsServer);


        verticalLayout_2->addLayout(horizontalLayout);


        hboxLayout->addWidget(frame);


        vboxLayout->addWidget(grpWmsServers);

        buttonBox = new QDialogButtonBox(WMSPreferencesDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Apply|QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(WMSPreferencesDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, WMSPreferencesDialog, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, WMSPreferencesDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(WMSPreferencesDialog);
    } // setupUi

    void retranslateUi(QDialog *WMSPreferencesDialog)
    {
        WMSPreferencesDialog->setWindowTitle(QCoreApplication::translate("WMSPreferencesDialog", "WMS servers setup", nullptr));
        grpWmsServers->setTitle(QCoreApplication::translate("WMSPreferencesDialog", "WMS Servers", nullptr));
        label_10->setText(QCoreApplication::translate("WMSPreferencesDialog", "Server list:", nullptr));
        label_11->setText(QCoreApplication::translate("WMSPreferencesDialog", "Name:", nullptr));
        label_5->setText(QCoreApplication::translate("WMSPreferencesDialog", "Server Url:", nullptr));
        label_7->setText(QCoreApplication::translate("WMSPreferencesDialog", "Layers:", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = tvWmsLayers->headerItem();
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("WMSPreferencesDialog", "Title", nullptr));
        label_8->setText(QCoreApplication::translate("WMSPreferencesDialog", "Projection:", nullptr));
        cbTileIt->setText(QCoreApplication::translate("WMSPreferencesDialog", "Tile it", nullptr));
        label_2->setText(QCoreApplication::translate("WMSPreferencesDialog", "Zoom levels", nullptr));
        label->setText(QCoreApplication::translate("WMSPreferencesDialog", "Image format:", nullptr));
        label_9->setText(QCoreApplication::translate("WMSPreferencesDialog", "Styles:", nullptr));
        label_3->setText(QCoreApplication::translate("WMSPreferencesDialog", "\"source\" tag value:", nullptr));
        label_4->setText(QCoreApplication::translate("WMSPreferencesDialog", "License URL:", nullptr));
        btApplyWmsServer->setText(QCoreApplication::translate("WMSPreferencesDialog", "Apply", nullptr));
        btAddWmsServer->setText(QCoreApplication::translate("WMSPreferencesDialog", "Add", nullptr));
        btDelWmsServer->setText(QCoreApplication::translate("WMSPreferencesDialog", "Remove", nullptr));
    } // retranslateUi

};

namespace Ui {
    class WMSPreferencesDialog: public Ui_WMSPreferencesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WMSPREFERENCESDIALOG_H
