/********************************************************************************
** Form generated from reading UI file 'ProjectionChooser.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROJECTIONCHOOSER_H
#define UI_PROJECTIONCHOOSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ProjectionChooser
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QRadioButton *chkPredefined;
    QRadioButton *chkStandard;
    QLineEdit *txtStandard;
    QComboBox *cbPredefined;
    QRadioButton *chkCustom;
    QLineEdit *txtCustom;
    QRadioButton *chkWkt;
    QTextEdit *txWkt;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ProjectionChooser)
    {
        if (ProjectionChooser->objectName().isEmpty())
            ProjectionChooser->setObjectName("ProjectionChooser");
        ProjectionChooser->resize(500, 227);
        ProjectionChooser->setMinimumSize(QSize(500, 0));
        verticalLayout = new QVBoxLayout(ProjectionChooser);
        verticalLayout->setObjectName("verticalLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        chkPredefined = new QRadioButton(ProjectionChooser);
        chkPredefined->setObjectName("chkPredefined");

        gridLayout->addWidget(chkPredefined, 0, 0, 1, 1);

        chkStandard = new QRadioButton(ProjectionChooser);
        chkStandard->setObjectName("chkStandard");

        gridLayout->addWidget(chkStandard, 2, 0, 1, 1);

        txtStandard = new QLineEdit(ProjectionChooser);
        txtStandard->setObjectName("txtStandard");

        gridLayout->addWidget(txtStandard, 2, 1, 1, 1);

        cbPredefined = new QComboBox(ProjectionChooser);
        cbPredefined->setObjectName("cbPredefined");
        QSizePolicy sizePolicy(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(cbPredefined->sizePolicy().hasHeightForWidth());
        cbPredefined->setSizePolicy(sizePolicy);
        cbPredefined->setMaximumSize(QSize(500, 16777215));

        gridLayout->addWidget(cbPredefined, 0, 1, 1, 1);

        chkCustom = new QRadioButton(ProjectionChooser);
        chkCustom->setObjectName("chkCustom");

        gridLayout->addWidget(chkCustom, 1, 0, 1, 1);

        txtCustom = new QLineEdit(ProjectionChooser);
        txtCustom->setObjectName("txtCustom");

        gridLayout->addWidget(txtCustom, 1, 1, 1, 1);

        chkWkt = new QRadioButton(ProjectionChooser);
        chkWkt->setObjectName("chkWkt");

        gridLayout->addWidget(chkWkt, 3, 0, 1, 1);

        txWkt = new QTextEdit(ProjectionChooser);
        txWkt->setObjectName("txWkt");
        txWkt->setMinimumSize(QSize(0, 100));
        txWkt->setMaximumSize(QSize(16777215, 100));

        gridLayout->addWidget(txWkt, 3, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        buttonBox = new QDialogButtonBox(ProjectionChooser);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(ProjectionChooser);
        QObject::connect(buttonBox, &QDialogButtonBox::accepted, ProjectionChooser, qOverload<>(&QDialog::accept));
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, ProjectionChooser, qOverload<>(&QDialog::reject));
        QObject::connect(cbPredefined, &QComboBox::currentIndexChanged, chkPredefined, qOverload<>(&QRadioButton::toggle));
        QObject::connect(txtStandard, &QLineEdit::textEdited, chkStandard, qOverload<>(&QRadioButton::toggle));
        QObject::connect(txtCustom, &QLineEdit::textEdited, chkCustom, qOverload<>(&QRadioButton::toggle));
        QObject::connect(txWkt, &QTextEdit::textChanged, chkWkt, qOverload<>(&QRadioButton::toggle));

        QMetaObject::connectSlotsByName(ProjectionChooser);
    } // setupUi

    void retranslateUi(QDialog *ProjectionChooser)
    {
        ProjectionChooser->setWindowTitle(QCoreApplication::translate("ProjectionChooser", "Dialog", nullptr));
        chkPredefined->setText(QCoreApplication::translate("ProjectionChooser", "Predefined", nullptr));
        chkStandard->setText(QCoreApplication::translate("ProjectionChooser", "Standard (EPSG:...)", nullptr));
        chkCustom->setText(QCoreApplication::translate("ProjectionChooser", "Custom PROJ4(+proj=...)", nullptr));
        chkWkt->setText(QCoreApplication::translate("ProjectionChooser", "WKT", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ProjectionChooser: public Ui_ProjectionChooser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROJECTIONCHOOSER_H
