/********************************************************************************
** Form generated from reading UI file 'SelectionDialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SELECTIONDIALOG_H
#define UI_SELECTIONDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_SelectionDialog
{
public:
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QLabel *label_4;
    QLabel *label_3;
    QLabel *label;
    QLineEdit *edName;
    QComboBox *cbKey;
    QComboBox *cbValue;
    QFrame *line;
    QLineEdit *edID;
    QLabel *label_6;
    QLabel *label_7;
    QLineEdit *edTagQuery;
    QWidget *widgetMaxResult;
    QFormLayout *formLayout;
    QLabel *label_5;
    QFormLayout *layoutMaxResult;
    QSpinBox *sbMaxResult;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SelectionDialog)
    {
        if (SelectionDialog->objectName().isEmpty())
            SelectionDialog->setObjectName("SelectionDialog");
        SelectionDialog->resize(511, 246);
        verticalLayout_2 = new QVBoxLayout(SelectionDialog);
        verticalLayout_2->setObjectName("verticalLayout_2");
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(0);
        gridLayout->setObjectName("gridLayout");
        label_2 = new QLabel(SelectionDialog);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        label_4 = new QLabel(SelectionDialog);
        label_4->setObjectName("label_4");

        gridLayout->addWidget(label_4, 2, 2, 1, 1);

        label_3 = new QLabel(SelectionDialog);
        label_3->setObjectName("label_3");
        label_3->setMaximumSize(QSize(50, 16777215));
        label_3->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(label_3, 3, 1, 1, 1);

        label = new QLabel(SelectionDialog);
        label->setObjectName("label");

        gridLayout->addWidget(label, 0, 0, 1, 1);

        edName = new QLineEdit(SelectionDialog);
        edName->setObjectName("edName");

        gridLayout->addWidget(edName, 1, 0, 1, 1);

        cbKey = new QComboBox(SelectionDialog);
        cbKey->setObjectName("cbKey");

        gridLayout->addWidget(cbKey, 3, 0, 1, 1);

        cbValue = new QComboBox(SelectionDialog);
        cbValue->setObjectName("cbValue");

        gridLayout->addWidget(cbValue, 3, 2, 1, 1);

        line = new QFrame(SelectionDialog);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line, 4, 0, 1, 3);

        edID = new QLineEdit(SelectionDialog);
        edID->setObjectName("edID");

        gridLayout->addWidget(edID, 1, 2, 1, 1);

        label_6 = new QLabel(SelectionDialog);
        label_6->setObjectName("label_6");

        gridLayout->addWidget(label_6, 0, 2, 1, 1);

        label_7 = new QLabel(SelectionDialog);
        label_7->setObjectName("label_7");

        gridLayout->addWidget(label_7, 5, 0, 1, 1);

        edTagQuery = new QLineEdit(SelectionDialog);
        edTagQuery->setObjectName("edTagQuery");

        gridLayout->addWidget(edTagQuery, 6, 0, 1, 3);


        verticalLayout_2->addLayout(gridLayout);

        widgetMaxResult = new QWidget(SelectionDialog);
        widgetMaxResult->setObjectName("widgetMaxResult");
        formLayout = new QFormLayout(widgetMaxResult);
        formLayout->setContentsMargins(0, 0, 0, 0);
        formLayout->setObjectName("formLayout");
        label_5 = new QLabel(widgetMaxResult);
        label_5->setObjectName("label_5");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, label_5);

        layoutMaxResult = new QFormLayout();
        layoutMaxResult->setObjectName("layoutMaxResult");
        layoutMaxResult->setFieldGrowthPolicy(QFormLayout::AllNonFixedFieldsGrow);
        sbMaxResult = new QSpinBox(widgetMaxResult);
        sbMaxResult->setObjectName("sbMaxResult");
        sbMaxResult->setMinimum(0);
        sbMaxResult->setMaximum(99999);
        sbMaxResult->setValue(999);

        layoutMaxResult->setWidget(0, QFormLayout::ItemRole::LabelRole, sbMaxResult);


        formLayout->setLayout(1, QFormLayout::ItemRole::LabelRole, layoutMaxResult);


        verticalLayout_2->addWidget(widgetMaxResult);

        buttonBox = new QDialogButtonBox(SelectionDialog);
        buttonBox->setObjectName("buttonBox");
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_2->addWidget(buttonBox);

        QWidget::setTabOrder(buttonBox, edName);
        QWidget::setTabOrder(edName, edID);
        QWidget::setTabOrder(edID, cbKey);
        QWidget::setTabOrder(cbKey, cbValue);

        retranslateUi(SelectionDialog);
        QObject::connect(buttonBox, &QDialogButtonBox::rejected, SelectionDialog, qOverload<>(&QDialog::reject));

        QMetaObject::connectSlotsByName(SelectionDialog);
    } // setupUi

    void retranslateUi(QDialog *SelectionDialog)
    {
        SelectionDialog->setWindowTitle(QCoreApplication::translate("SelectionDialog", "Selection", nullptr));
        label_2->setText(QCoreApplication::translate("SelectionDialog", "Key", nullptr));
        label_4->setText(QCoreApplication::translate("SelectionDialog", "Value", nullptr));
        label_3->setText(QCoreApplication::translate("SelectionDialog", "=", nullptr));
        label->setText(QCoreApplication::translate("SelectionDialog", "Name", nullptr));
        label_6->setText(QCoreApplication::translate("SelectionDialog", "Id", nullptr));
        label_7->setText(QCoreApplication::translate("SelectionDialog", "Query", nullptr));
        label_5->setText(QCoreApplication::translate("SelectionDialog", "Maximum returned results (0 for all)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SelectionDialog: public Ui_SelectionDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SELECTIONDIALOG_H
