/****************************************************************************
** Meta object code from reading C++ file 'MainWindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../merkaartor/src/MainWindow.h"
#include <QtGui/qtextcursor.h>
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'MainWindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10MainWindowE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindow::qt_create_metaobjectdata<qt_meta_tag_ZN10MainWindowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindow",
        "remove_triggered",
        "",
        "move_triggered",
        "add_triggered",
        "reverse_triggered",
        "content_changed",
        "invalidateView",
        "UpdateDock",
        "toggleButtonStyle",
        "on_createRoundaboutAction_triggered",
        "on_createPolygonAction_triggered",
        "on_createRectangleAction_triggered",
        "on_createDoubleWayAction_triggered",
        "on_createNodeAction_triggered",
        "on_markBridgeAction_triggered",
        "on_createRoadAction_triggered",
        "on_createCurvedRoadAction_triggered",
        "on_createRelationAction_triggered",
        "on_createAreaAction_triggered",
        "on_editPropertiesAction_triggered",
        "on_editUndoAction_triggered",
        "on_editRedoAction_triggered",
        "on_editMapStyleAction_triggered",
        "on_editCutAction_triggered",
        "on_editCopyAction_triggered",
        "on_editPasteFeatureAction_triggered",
        "on_editPasteOverwriteAction_triggered",
        "on_editPasteMergeAction_triggered",
        "on_layersNewImageAction_triggered",
        "on_layersNewDrawingAction_triggered",
        "on_layersNewFilterAction_triggered",
        "on_fileNewAction_triggered",
        "on_fileDownloadAction_triggered",
        "on_fileDownloadMoreAction_triggered",
        "on_fileUploadAction_triggered",
        "on_fileImportAction_triggered",
        "on_fileImportGDALAction_triggered",
        "on_fileOpenAction_triggered",
        "on_fileSaveAsAction_triggered",
        "on_fileSaveAsTemplateAction_triggered",
        "on_fileSaveAction_triggered",
        "on_fileWorkOfflineAction_triggered",
        "on_filePrintAction_triggered",
        "on_filePropertiesAction_triggered",
        "on_helpAboutAction_triggered",
        "on_viewZoomAllAction_triggered",
        "on_viewZoomInAction_triggered",
        "on_viewZoomOutAction_triggered",
        "on_viewZoomWindowAction_triggered",
        "on_viewLockZoomAction_triggered",
        "on_viewDownloadedAction_triggered",
        "on_viewDirtyAction_triggered",
        "on_viewScaleAction_triggered",
        "on_viewPhotosAction_triggered",
        "on_viewShowLatLonGridAction_triggered",
        "on_viewStyleBackgroundAction_triggered",
        "on_viewStyleForegroundAction_triggered",
        "on_viewStyleTouchupAction_triggered",
        "on_viewNamesAction_triggered",
        "on_viewVirtualNodesAction_triggered",
        "on_viewTrackPointsAction_triggered",
        "on_viewTrackSegmentsAction_triggered",
        "on_viewRelationsAction_triggered",
        "on_viewGotoAction_triggered",
        "on_viewArrowsNeverAction_triggered",
        "checked",
        "on_viewArrowsOnewayAction_triggered",
        "on_viewArrowsAlwaysAction_triggered",
        "on_editRemoveAction_triggered",
        "on_editMoveAction_triggered",
        "on_editRotateAction_triggered",
        "on_editScaleAction_triggered",
        "on_editReverseAction_triggered",
        "on_roadSplitAction_triggered",
        "on_roadBreakAction_triggered",
        "on_roadJoinAction_triggered",
        "on_roadSimplifyAction_triggered",
        "on_roadCreateJunctionAction_triggered",
        "on_roadAddStreetNumbersAction_triggered",
        "on_roadSubdivideAction_triggered",
        "on_roadAxisAlignAction_triggered",
        "on_roadExtrudeAction_triggered",
        "on_roadBingExtractAction_triggered",
        "on_featureSelectChildrenAction_triggered",
        "on_featureSelectParentsAction_triggered",
        "on_featureDownloadMissingChildrenAction_triggered",
        "on_featureDeleteAction_triggered",
        "on_featureCommitAction_triggered",
        "on_nodeAlignAction_triggered",
        "on_nodeSpreadAction_triggered",
        "on_nodeMergeAction_triggered",
        "on_nodeDetachAction_triggered",
        "on_relationAddMemberAction_triggered",
        "on_relationRemoveMemberAction_triggered",
        "on_relationAddToMultipolygonAction_triggered",
        "on_areaJoinAction_triggered",
        "on_areaSplitAction_triggered",
        "on_areaTerraceAction_triggered",
        "on_mapStyleSaveAction_triggered",
        "on_mapStyleSaveAsAction_triggered",
        "on_mapStyleLoadAction_triggered",
        "on_exportOSMAction_triggered",
        "on_exportOSCAction_triggered",
        "on_exportGPXAction_triggered",
        "on_exportKMLAction_triggered",
        "on_exportGDALAction_triggered",
        "on_editSelectAction_triggered",
        "on_bookmarkAddAction_triggered",
        "on_bookmarkRemoveAction_triggered",
        "on_toolsPreferencesAction_triggered",
        "on_toolsShortcutsAction_triggered",
        "on_toolsToolbarsAction_triggered",
        "on_toolsWMSServersAction_triggered",
        "on_toolsTMSServersAction_triggered",
        "on_toolsProjectionsAction_triggered",
        "on_toolsFiltersAction_triggered",
        "toolsPreferencesAction_triggered",
        "focusData",
        "on_toolsResetDiscardableAction_triggered",
        "on_toolsRebuildHistoryAction_triggered",
        "on_windowPropertiesAction_triggered",
        "on_windowLayersAction_triggered",
        "on_windowInfoAction_triggered",
        "on_windowDirtyAction_triggered",
        "on_windowFeatsAction_triggered",
        "on_windowToolbarAction_triggered",
        "on_windowGPSAction_triggered",
        "on_windowGeoimageAction_triggered",
        "on_windowStylesAction_triggered",
        "on_windowHideAllAction_triggered",
        "on_windowShowAllAction_triggered",
        "on_gpsConnectAction_triggered",
        "on_gpsReplayAction_triggered",
        "on_gpsRecordAction_triggered",
        "on_gpsPauseAction_triggered",
        "on_gpsDisconnectAction_triggered",
        "on_gpsCenterAction_triggered",
        "preferencesChanged",
        "PreferencesDialog*",
        "prefs",
        "clipboardChanged",
        "on_toolTemplatesSaveAction_triggered",
        "on_toolTemplatesMergeAction_triggered",
        "on_toolTemplatesLoadAction_triggered",
        "onCustomcontextmenurequested",
        "QPoint",
        "pos",
        "onImagerequested",
        "ImageMapLayer*",
        "onImagereceived",
        "aLayer",
        "onLoadingfinished",
        "handleMessage",
        "msg",
        "adjustLayers",
        "adjustViewport",
        "bookmarkTriggered",
        "QAction*",
        "anAction",
        "recentOpenTriggered",
        "recentImportTriggered",
        "projectionSet",
        "prj",
        "projectionTriggered",
        "styleTriggered",
        "updateGpsPosition",
        "latitude",
        "longitude",
        "time",
        "altitude",
        "speed",
        "heading",
        "applyStyles",
        "NewStyle",
        "applyPainters",
        "GlobalPainter*",
        "theGlobalPainter",
        "QList<Painter>*",
        "thePainters",
        "delayedInit",
        "setAreaOpacity",
        "updateBookmarksMenu",
        "updateWindowMenu",
        "b",
        "on_viewWireframeAction_toggled",
        "arg1"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'remove_triggered'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'move_triggered'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'add_triggered'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'reverse_triggered'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'content_changed'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'invalidateView'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'invalidateView'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'toggleButtonStyle'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createRoundaboutAction_triggered'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createPolygonAction_triggered'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createRectangleAction_triggered'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createDoubleWayAction_triggered'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createNodeAction_triggered'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_markBridgeAction_triggered'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createRoadAction_triggered'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createCurvedRoadAction_triggered'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createRelationAction_triggered'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_createAreaAction_triggered'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editPropertiesAction_triggered'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editUndoAction_triggered'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editRedoAction_triggered'
        QtMocHelpers::SlotData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editMapStyleAction_triggered'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editCutAction_triggered'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editCopyAction_triggered'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editPasteFeatureAction_triggered'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editPasteOverwriteAction_triggered'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editPasteMergeAction_triggered'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_layersNewImageAction_triggered'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_layersNewDrawingAction_triggered'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_layersNewFilterAction_triggered'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileNewAction_triggered'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileDownloadAction_triggered'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileDownloadMoreAction_triggered'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileUploadAction_triggered'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileImportAction_triggered'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileImportGDALAction_triggered'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileOpenAction_triggered'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileSaveAsAction_triggered'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileSaveAsTemplateAction_triggered'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileSaveAction_triggered'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_fileWorkOfflineAction_triggered'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_filePrintAction_triggered'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_filePropertiesAction_triggered'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_helpAboutAction_triggered'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewZoomAllAction_triggered'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewZoomInAction_triggered'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewZoomOutAction_triggered'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewZoomWindowAction_triggered'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewLockZoomAction_triggered'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewDownloadedAction_triggered'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewDirtyAction_triggered'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewScaleAction_triggered'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewPhotosAction_triggered'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewShowLatLonGridAction_triggered'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewStyleBackgroundAction_triggered'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewStyleForegroundAction_triggered'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewStyleTouchupAction_triggered'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewNamesAction_triggered'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewVirtualNodesAction_triggered'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewTrackPointsAction_triggered'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewTrackSegmentsAction_triggered'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewRelationsAction_triggered'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewGotoAction_triggered'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_viewArrowsNeverAction_triggered'
        QtMocHelpers::SlotData<void(bool)>(65, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 66 },
        }}),
        // Slot 'on_viewArrowsOnewayAction_triggered'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 66 },
        }}),
        // Slot 'on_viewArrowsAlwaysAction_triggered'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 66 },
        }}),
        // Slot 'on_editRemoveAction_triggered'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editMoveAction_triggered'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editRotateAction_triggered'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editScaleAction_triggered'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editReverseAction_triggered'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadSplitAction_triggered'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadBreakAction_triggered'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadJoinAction_triggered'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadSimplifyAction_triggered'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadCreateJunctionAction_triggered'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadAddStreetNumbersAction_triggered'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadSubdivideAction_triggered'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadAxisAlignAction_triggered'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadExtrudeAction_triggered'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_roadBingExtractAction_triggered'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_featureSelectChildrenAction_triggered'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_featureSelectParentsAction_triggered'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_featureDownloadMissingChildrenAction_triggered'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_featureDeleteAction_triggered'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_featureCommitAction_triggered'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_nodeAlignAction_triggered'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_nodeSpreadAction_triggered'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_nodeMergeAction_triggered'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_nodeDetachAction_triggered'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_relationAddMemberAction_triggered'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_relationRemoveMemberAction_triggered'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_relationAddToMultipolygonAction_triggered'
        QtMocHelpers::SlotData<void()>(95, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_areaJoinAction_triggered'
        QtMocHelpers::SlotData<void()>(96, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_areaSplitAction_triggered'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_areaTerraceAction_triggered'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_mapStyleSaveAction_triggered'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_mapStyleSaveAsAction_triggered'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_mapStyleLoadAction_triggered'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_exportOSMAction_triggered'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_exportOSCAction_triggered'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_exportGPXAction_triggered'
        QtMocHelpers::SlotData<void()>(104, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_exportKMLAction_triggered'
        QtMocHelpers::SlotData<void()>(105, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_exportGDALAction_triggered'
        QtMocHelpers::SlotData<void()>(106, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_editSelectAction_triggered'
        QtMocHelpers::SlotData<void()>(107, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_bookmarkAddAction_triggered'
        QtMocHelpers::SlotData<void()>(108, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_bookmarkRemoveAction_triggered'
        QtMocHelpers::SlotData<void()>(109, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsPreferencesAction_triggered'
        QtMocHelpers::SlotData<void()>(110, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsShortcutsAction_triggered'
        QtMocHelpers::SlotData<void()>(111, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsToolbarsAction_triggered'
        QtMocHelpers::SlotData<void()>(112, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsWMSServersAction_triggered'
        QtMocHelpers::SlotData<void()>(113, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsTMSServersAction_triggered'
        QtMocHelpers::SlotData<void()>(114, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsProjectionsAction_triggered'
        QtMocHelpers::SlotData<void()>(115, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsFiltersAction_triggered'
        QtMocHelpers::SlotData<void()>(116, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'toolsPreferencesAction_triggered'
        QtMocHelpers::SlotData<void(bool)>(117, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 118 },
        }}),
        // Slot 'toolsPreferencesAction_triggered'
        QtMocHelpers::SlotData<void()>(117, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'on_toolsResetDiscardableAction_triggered'
        QtMocHelpers::SlotData<void()>(119, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolsRebuildHistoryAction_triggered'
        QtMocHelpers::SlotData<void()>(120, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowPropertiesAction_triggered'
        QtMocHelpers::SlotData<void()>(121, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowLayersAction_triggered'
        QtMocHelpers::SlotData<void()>(122, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowInfoAction_triggered'
        QtMocHelpers::SlotData<void()>(123, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowDirtyAction_triggered'
        QtMocHelpers::SlotData<void()>(124, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowFeatsAction_triggered'
        QtMocHelpers::SlotData<void()>(125, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowToolbarAction_triggered'
        QtMocHelpers::SlotData<void()>(126, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowGPSAction_triggered'
        QtMocHelpers::SlotData<void()>(127, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowGeoimageAction_triggered'
        QtMocHelpers::SlotData<void()>(128, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowStylesAction_triggered'
        QtMocHelpers::SlotData<void()>(129, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowHideAllAction_triggered'
        QtMocHelpers::SlotData<void()>(130, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_windowShowAllAction_triggered'
        QtMocHelpers::SlotData<void()>(131, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gpsConnectAction_triggered'
        QtMocHelpers::SlotData<void()>(132, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gpsReplayAction_triggered'
        QtMocHelpers::SlotData<void()>(133, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gpsRecordAction_triggered'
        QtMocHelpers::SlotData<void()>(134, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gpsPauseAction_triggered'
        QtMocHelpers::SlotData<void()>(135, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gpsDisconnectAction_triggered'
        QtMocHelpers::SlotData<void()>(136, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_gpsCenterAction_triggered'
        QtMocHelpers::SlotData<void()>(137, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'preferencesChanged'
        QtMocHelpers::SlotData<void(PreferencesDialog *)>(138, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 139, 140 },
        }}),
        // Slot 'clipboardChanged'
        QtMocHelpers::SlotData<void()>(141, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolTemplatesSaveAction_triggered'
        QtMocHelpers::SlotData<void()>(142, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolTemplatesMergeAction_triggered'
        QtMocHelpers::SlotData<void()>(143, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_toolTemplatesLoadAction_triggered'
        QtMocHelpers::SlotData<void()>(144, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onCustomcontextmenurequested'
        QtMocHelpers::SlotData<void(const QPoint &)>(145, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 146, 147 },
        }}),
        // Slot 'onImagerequested'
        QtMocHelpers::SlotData<void(ImageMapLayer *)>(148, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 149, 2 },
        }}),
        // Slot 'onImagereceived'
        QtMocHelpers::SlotData<void(ImageMapLayer *)>(150, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 149, 151 },
        }}),
        // Slot 'onLoadingfinished'
        QtMocHelpers::SlotData<void(ImageMapLayer *)>(152, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 149, 2 },
        }}),
        // Slot 'handleMessage'
        QtMocHelpers::SlotData<void(const QString &)>(153, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 154 },
        }}),
        // Slot 'adjustLayers'
        QtMocHelpers::SlotData<void(bool)>(155, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 156 },
        }}),
        // Slot 'bookmarkTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(157, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 158, 159 },
        }}),
        // Slot 'recentOpenTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(160, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 158, 159 },
        }}),
        // Slot 'recentImportTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(161, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 158, 159 },
        }}),
        // Slot 'projectionSet'
        QtMocHelpers::SlotData<void(const QString &)>(162, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 163 },
        }}),
        // Slot 'projectionTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(164, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 158, 159 },
        }}),
        // Slot 'styleTriggered'
        QtMocHelpers::SlotData<void(QAction *)>(165, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 158, 159 },
        }}),
        // Slot 'updateGpsPosition'
        QtMocHelpers::SlotData<void(qreal, qreal, QDateTime, qreal, qreal, qreal)>(166, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QReal, 167 }, { QMetaType::QReal, 168 }, { QMetaType::QDateTime, 169 }, { QMetaType::QReal, 170 },
            { QMetaType::QReal, 171 }, { QMetaType::QReal, 172 },
        }}),
        // Slot 'applyStyles'
        QtMocHelpers::SlotData<void(QString)>(173, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 174 },
        }}),
        // Slot 'applyPainters'
        QtMocHelpers::SlotData<void(GlobalPainter *, QList<Painter> *)>(175, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 176, 177 }, { 0x80000000 | 178, 179 },
        }}),
        // Slot 'delayedInit'
        QtMocHelpers::SlotData<void()>(180, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setAreaOpacity'
        QtMocHelpers::SlotData<void(QAction *)>(181, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 158, 2 },
        }}),
        // Slot 'updateBookmarksMenu'
        QtMocHelpers::SlotData<void()>(182, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateWindowMenu'
        QtMocHelpers::SlotData<void(bool)>(183, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 184 },
        }}),
        // Slot 'updateWindowMenu'
        QtMocHelpers::SlotData<void()>(183, 2, QMC::AccessPrivate | QMC::MethodCloned, QMetaType::Void),
        // Slot 'on_viewWireframeAction_toggled'
        QtMocHelpers::SlotData<void(bool)>(185, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 186 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindow, qt_meta_tag_ZN10MainWindowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10MainWindowE_t>.metaTypes,
    nullptr
} };

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->remove_triggered(); break;
        case 1: _t->move_triggered(); break;
        case 2: _t->add_triggered(); break;
        case 3: _t->reverse_triggered(); break;
        case 4: _t->content_changed(); break;
        case 5: _t->invalidateView((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->invalidateView(); break;
        case 7: _t->toggleButtonStyle(); break;
        case 8: _t->on_createRoundaboutAction_triggered(); break;
        case 9: _t->on_createPolygonAction_triggered(); break;
        case 10: _t->on_createRectangleAction_triggered(); break;
        case 11: _t->on_createDoubleWayAction_triggered(); break;
        case 12: _t->on_createNodeAction_triggered(); break;
        case 13: _t->on_markBridgeAction_triggered(); break;
        case 14: _t->on_createRoadAction_triggered(); break;
        case 15: _t->on_createCurvedRoadAction_triggered(); break;
        case 16: _t->on_createRelationAction_triggered(); break;
        case 17: _t->on_createAreaAction_triggered(); break;
        case 18: _t->on_editPropertiesAction_triggered(); break;
        case 19: _t->on_editUndoAction_triggered(); break;
        case 20: _t->on_editRedoAction_triggered(); break;
        case 21: _t->on_editMapStyleAction_triggered(); break;
        case 22: _t->on_editCutAction_triggered(); break;
        case 23: _t->on_editCopyAction_triggered(); break;
        case 24: _t->on_editPasteFeatureAction_triggered(); break;
        case 25: _t->on_editPasteOverwriteAction_triggered(); break;
        case 26: _t->on_editPasteMergeAction_triggered(); break;
        case 27: _t->on_layersNewImageAction_triggered(); break;
        case 28: _t->on_layersNewDrawingAction_triggered(); break;
        case 29: _t->on_layersNewFilterAction_triggered(); break;
        case 30: _t->on_fileNewAction_triggered(); break;
        case 31: _t->on_fileDownloadAction_triggered(); break;
        case 32: _t->on_fileDownloadMoreAction_triggered(); break;
        case 33: _t->on_fileUploadAction_triggered(); break;
        case 34: _t->on_fileImportAction_triggered(); break;
        case 35: _t->on_fileImportGDALAction_triggered(); break;
        case 36: _t->on_fileOpenAction_triggered(); break;
        case 37: _t->on_fileSaveAsAction_triggered(); break;
        case 38: _t->on_fileSaveAsTemplateAction_triggered(); break;
        case 39: _t->on_fileSaveAction_triggered(); break;
        case 40: _t->on_fileWorkOfflineAction_triggered(); break;
        case 41: _t->on_filePrintAction_triggered(); break;
        case 42: _t->on_filePropertiesAction_triggered(); break;
        case 43: _t->on_helpAboutAction_triggered(); break;
        case 44: _t->on_viewZoomAllAction_triggered(); break;
        case 45: _t->on_viewZoomInAction_triggered(); break;
        case 46: _t->on_viewZoomOutAction_triggered(); break;
        case 47: _t->on_viewZoomWindowAction_triggered(); break;
        case 48: _t->on_viewLockZoomAction_triggered(); break;
        case 49: _t->on_viewDownloadedAction_triggered(); break;
        case 50: _t->on_viewDirtyAction_triggered(); break;
        case 51: _t->on_viewScaleAction_triggered(); break;
        case 52: _t->on_viewPhotosAction_triggered(); break;
        case 53: _t->on_viewShowLatLonGridAction_triggered(); break;
        case 54: _t->on_viewStyleBackgroundAction_triggered(); break;
        case 55: _t->on_viewStyleForegroundAction_triggered(); break;
        case 56: _t->on_viewStyleTouchupAction_triggered(); break;
        case 57: _t->on_viewNamesAction_triggered(); break;
        case 58: _t->on_viewVirtualNodesAction_triggered(); break;
        case 59: _t->on_viewTrackPointsAction_triggered(); break;
        case 60: _t->on_viewTrackSegmentsAction_triggered(); break;
        case 61: _t->on_viewRelationsAction_triggered(); break;
        case 62: _t->on_viewGotoAction_triggered(); break;
        case 63: _t->on_viewArrowsNeverAction_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 64: _t->on_viewArrowsOnewayAction_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 65: _t->on_viewArrowsAlwaysAction_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 66: _t->on_editRemoveAction_triggered(); break;
        case 67: _t->on_editMoveAction_triggered(); break;
        case 68: _t->on_editRotateAction_triggered(); break;
        case 69: _t->on_editScaleAction_triggered(); break;
        case 70: _t->on_editReverseAction_triggered(); break;
        case 71: _t->on_roadSplitAction_triggered(); break;
        case 72: _t->on_roadBreakAction_triggered(); break;
        case 73: _t->on_roadJoinAction_triggered(); break;
        case 74: _t->on_roadSimplifyAction_triggered(); break;
        case 75: _t->on_roadCreateJunctionAction_triggered(); break;
        case 76: _t->on_roadAddStreetNumbersAction_triggered(); break;
        case 77: _t->on_roadSubdivideAction_triggered(); break;
        case 78: _t->on_roadAxisAlignAction_triggered(); break;
        case 79: _t->on_roadExtrudeAction_triggered(); break;
        case 80: _t->on_roadBingExtractAction_triggered(); break;
        case 81: _t->on_featureSelectChildrenAction_triggered(); break;
        case 82: _t->on_featureSelectParentsAction_triggered(); break;
        case 83: _t->on_featureDownloadMissingChildrenAction_triggered(); break;
        case 84: _t->on_featureDeleteAction_triggered(); break;
        case 85: _t->on_featureCommitAction_triggered(); break;
        case 86: _t->on_nodeAlignAction_triggered(); break;
        case 87: _t->on_nodeSpreadAction_triggered(); break;
        case 88: _t->on_nodeMergeAction_triggered(); break;
        case 89: _t->on_nodeDetachAction_triggered(); break;
        case 90: _t->on_relationAddMemberAction_triggered(); break;
        case 91: _t->on_relationRemoveMemberAction_triggered(); break;
        case 92: _t->on_relationAddToMultipolygonAction_triggered(); break;
        case 93: _t->on_areaJoinAction_triggered(); break;
        case 94: _t->on_areaSplitAction_triggered(); break;
        case 95: _t->on_areaTerraceAction_triggered(); break;
        case 96: _t->on_mapStyleSaveAction_triggered(); break;
        case 97: _t->on_mapStyleSaveAsAction_triggered(); break;
        case 98: _t->on_mapStyleLoadAction_triggered(); break;
        case 99: _t->on_exportOSMAction_triggered(); break;
        case 100: _t->on_exportOSCAction_triggered(); break;
        case 101: _t->on_exportGPXAction_triggered(); break;
        case 102: _t->on_exportKMLAction_triggered(); break;
        case 103: _t->on_exportGDALAction_triggered(); break;
        case 104: _t->on_editSelectAction_triggered(); break;
        case 105: _t->on_bookmarkAddAction_triggered(); break;
        case 106: _t->on_bookmarkRemoveAction_triggered(); break;
        case 107: _t->on_toolsPreferencesAction_triggered(); break;
        case 108: _t->on_toolsShortcutsAction_triggered(); break;
        case 109: _t->on_toolsToolbarsAction_triggered(); break;
        case 110: _t->on_toolsWMSServersAction_triggered(); break;
        case 111: _t->on_toolsTMSServersAction_triggered(); break;
        case 112: _t->on_toolsProjectionsAction_triggered(); break;
        case 113: _t->on_toolsFiltersAction_triggered(); break;
        case 114: _t->toolsPreferencesAction_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 115: _t->toolsPreferencesAction_triggered(); break;
        case 116: _t->on_toolsResetDiscardableAction_triggered(); break;
        case 117: _t->on_toolsRebuildHistoryAction_triggered(); break;
        case 118: _t->on_windowPropertiesAction_triggered(); break;
        case 119: _t->on_windowLayersAction_triggered(); break;
        case 120: _t->on_windowInfoAction_triggered(); break;
        case 121: _t->on_windowDirtyAction_triggered(); break;
        case 122: _t->on_windowFeatsAction_triggered(); break;
        case 123: _t->on_windowToolbarAction_triggered(); break;
        case 124: _t->on_windowGPSAction_triggered(); break;
        case 125: _t->on_windowGeoimageAction_triggered(); break;
        case 126: _t->on_windowStylesAction_triggered(); break;
        case 127: _t->on_windowHideAllAction_triggered(); break;
        case 128: _t->on_windowShowAllAction_triggered(); break;
        case 129: _t->on_gpsConnectAction_triggered(); break;
        case 130: _t->on_gpsReplayAction_triggered(); break;
        case 131: _t->on_gpsRecordAction_triggered(); break;
        case 132: _t->on_gpsPauseAction_triggered(); break;
        case 133: _t->on_gpsDisconnectAction_triggered(); break;
        case 134: _t->on_gpsCenterAction_triggered(); break;
        case 135: _t->preferencesChanged((*reinterpret_cast<std::add_pointer_t<PreferencesDialog*>>(_a[1]))); break;
        case 136: _t->clipboardChanged(); break;
        case 137: _t->on_toolTemplatesSaveAction_triggered(); break;
        case 138: _t->on_toolTemplatesMergeAction_triggered(); break;
        case 139: _t->on_toolTemplatesLoadAction_triggered(); break;
        case 140: _t->onCustomcontextmenurequested((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 141: _t->onImagerequested((*reinterpret_cast<std::add_pointer_t<ImageMapLayer*>>(_a[1]))); break;
        case 142: _t->onImagereceived((*reinterpret_cast<std::add_pointer_t<ImageMapLayer*>>(_a[1]))); break;
        case 143: _t->onLoadingfinished((*reinterpret_cast<std::add_pointer_t<ImageMapLayer*>>(_a[1]))); break;
        case 144: _t->handleMessage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 145: _t->adjustLayers((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 146: _t->bookmarkTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 147: _t->recentOpenTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 148: _t->recentImportTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 149: _t->projectionSet((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 150: _t->projectionTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 151: _t->styleTriggered((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 152: _t->updateGpsPosition((*reinterpret_cast<std::add_pointer_t<qreal>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[5])),(*reinterpret_cast<std::add_pointer_t<qreal>>(_a[6]))); break;
        case 153: _t->applyStyles((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 154: _t->applyPainters((*reinterpret_cast<std::add_pointer_t<GlobalPainter*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Painter>*>>(_a[2]))); break;
        case 155: _t->delayedInit(); break;
        case 156: _t->setAreaOpacity((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 157: _t->updateBookmarksMenu(); break;
        case 158: _t->updateWindowMenu((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 159: _t->updateWindowMenu(); break;
        case 160: _t->on_viewWireframeAction_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 146:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 147:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 148:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 150:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 151:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 156:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::remove_triggered, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::move_triggered, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::add_triggered, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::reverse_triggered, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindow::*)()>(_a, &MainWindow::content_changed, 4))
            return;
    }
}

const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10MainWindowE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IMerkMainWindow"))
        return static_cast< IMerkMainWindow*>(this);
    if (!strcmp(_clname, "IProgressWindow"))
        return static_cast< IProgressWindow*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 161)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 161;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 161)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 161;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::remove_triggered()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void MainWindow::move_triggered()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void MainWindow::add_triggered()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void MainWindow::reverse_triggered()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void MainWindow::content_changed()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
