/****************************************************************************
** Meta object code from reading C++ file 'PaintStyleEditor.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../merkaartor/src/PaintStyle/PaintStyleEditor.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'PaintStyleEditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16PaintStyleEditorE_t {};
} // unnamed namespace

template <> constexpr inline auto PaintStyleEditor::qt_create_metaobjectdata<qt_meta_tag_ZN16PaintStyleEditorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PaintStyleEditor",
        "stylesApplied",
        "",
        "GlobalPainter*",
        "theGlobalPainter",
        "QList<Painter>*",
        "thePainters",
        "on_DrawGlobalBackground_clicked",
        "b",
        "on_GlobalBackgroundColor_clicked",
        "on_DrawGlobalNodes_clicked",
        "on_GlobalNodesColor_clicked",
        "on_GlobalNodesProportional_valueChanged",
        "on_GlobalNodesFixed_valueChanged",
        "on_PaintList_itemSelectionChanged",
        "on_TagSelection_editingFinished",
        "on_LowerZoomBoundary_valueChanged",
        "on_UpperZoomBoundary_valueChanged",
        "on_DrawBackground_clicked",
        "on_BackgroundColor_clicked",
        "on_ProportionalBackground_valueChanged",
        "on_FixedBackground_valueChanged",
        "on_BackgroundInterior_clicked",
        "on_BackgroundExterior_clicked",
        "on_DrawForeground_clicked",
        "on_DrawFillIcon_clicked",
        "on_ForegroundColor_clicked",
        "on_ProportionalForeground_valueChanged",
        "on_FixedForeground_valueChanged",
        "on_ForegroundDashed_clicked",
        "on_ForegroundDashOn_valueChanged",
        "on_ForegroundDashOff_valueChanged",
        "on_DrawTrafficDirectionMarks_clicked",
        "on_TrafficDirectionMarksColor_clicked",
        "on_DrawTouchup_clicked",
        "on_TouchupColor_clicked",
        "on_ProportionalTouchup_valueChanged",
        "on_FixedTouchup_valueChanged",
        "on_TouchupDashed_clicked",
        "on_TouchupDashOn_valueChanged",
        "on_TouchupDashOff_valueChanged",
        "on_DrawFill_clicked",
        "on_FillColor_clicked",
        "on_DrawIcon_clicked",
        "on_IconName_textEdited",
        "on_ProportionalIcon_valueChanged",
        "on_FixedIcon_valueChanged",
        "on_DrawLabel_clicked",
        "on_LabelColor_clicked",
        "on_LabelTag_textEdited",
        "on_ProportionalLabel_valueChanged",
        "on_FixedLabel_valueChanged",
        "on_DrawLabelBackground_clicked",
        "on_LabelBackgroundlColor_clicked",
        "on_LabelFont_currentFontChanged",
        "QFont",
        "font",
        "on_LabelBackgroundTag_textEdited",
        "on_AddButton_clicked",
        "on_RemoveButton_clicked",
        "on_DuplicateButton_clicked",
        "on_btUp_clicked",
        "on_btDown_clicked",
        "on_buttonBox_clicked",
        "QAbstractButton*",
        "button",
        "on_LabelHalo_clicked",
        "on_LabelArea_clicked",
        "on_edFilter_textChanged",
        "text",
        "on_btSelectorHelper_clicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'stylesApplied'
        QtMocHelpers::SignalData<void(GlobalPainter *, QList<Painter> *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'on_DrawGlobalBackground_clicked'
        QtMocHelpers::SlotData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_GlobalBackgroundColor_clicked'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawGlobalNodes_clicked'
        QtMocHelpers::SlotData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_GlobalNodesColor_clicked'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_GlobalNodesProportional_valueChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_GlobalNodesFixed_valueChanged'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_PaintList_itemSelectionChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_TagSelection_editingFinished'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_LowerZoomBoundary_valueChanged'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_UpperZoomBoundary_valueChanged'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawBackground_clicked'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_BackgroundColor_clicked'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ProportionalBackground_valueChanged'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_FixedBackground_valueChanged'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_BackgroundInterior_clicked'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_BackgroundExterior_clicked'
        QtMocHelpers::SlotData<void(bool)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_DrawForeground_clicked'
        QtMocHelpers::SlotData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_DrawFillIcon_clicked'
        QtMocHelpers::SlotData<void(bool)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_ForegroundColor_clicked'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ProportionalForeground_valueChanged'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_FixedForeground_valueChanged'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ForegroundDashed_clicked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ForegroundDashOn_valueChanged'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ForegroundDashOff_valueChanged'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawTrafficDirectionMarks_clicked'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_TrafficDirectionMarksColor_clicked'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawTouchup_clicked'
        QtMocHelpers::SlotData<void(bool)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_TouchupColor_clicked'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ProportionalTouchup_valueChanged'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_FixedTouchup_valueChanged'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_TouchupDashed_clicked'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_TouchupDashOn_valueChanged'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_TouchupDashOff_valueChanged'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawFill_clicked'
        QtMocHelpers::SlotData<void(bool)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_FillColor_clicked'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawIcon_clicked'
        QtMocHelpers::SlotData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_IconName_textEdited'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ProportionalIcon_valueChanged'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_FixedIcon_valueChanged'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawLabel_clicked'
        QtMocHelpers::SlotData<void(bool)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_LabelColor_clicked'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_LabelTag_textEdited'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_ProportionalLabel_valueChanged'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_FixedLabel_valueChanged'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DrawLabelBackground_clicked'
        QtMocHelpers::SlotData<void(bool)>(52, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_LabelBackgroundlColor_clicked'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_LabelFont_currentFontChanged'
        QtMocHelpers::SlotData<void(const QFont &)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'on_LabelBackgroundTag_textEdited'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_AddButton_clicked'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_RemoveButton_clicked'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_DuplicateButton_clicked'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_btUp_clicked'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_btDown_clicked'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'on_buttonBox_clicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 64, 65 },
        }}),
        // Slot 'on_LabelHalo_clicked'
        QtMocHelpers::SlotData<void(bool)>(66, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_LabelArea_clicked'
        QtMocHelpers::SlotData<void(bool)>(67, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Slot 'on_edFilter_textChanged'
        QtMocHelpers::SlotData<void(const QString &)>(68, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 69 },
        }}),
        // Slot 'on_btSelectorHelper_clicked'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PaintStyleEditor, qt_meta_tag_ZN16PaintStyleEditorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PaintStyleEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16PaintStyleEditorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16PaintStyleEditorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16PaintStyleEditorE_t>.metaTypes,
    nullptr
} };

void PaintStyleEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PaintStyleEditor *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->stylesApplied((*reinterpret_cast<std::add_pointer_t<GlobalPainter*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<Painter>*>>(_a[2]))); break;
        case 1: _t->on_DrawGlobalBackground_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->on_GlobalBackgroundColor_clicked(); break;
        case 3: _t->on_DrawGlobalNodes_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->on_GlobalNodesColor_clicked(); break;
        case 5: _t->on_GlobalNodesProportional_valueChanged(); break;
        case 6: _t->on_GlobalNodesFixed_valueChanged(); break;
        case 7: _t->on_PaintList_itemSelectionChanged(); break;
        case 8: _t->on_TagSelection_editingFinished(); break;
        case 9: _t->on_LowerZoomBoundary_valueChanged(); break;
        case 10: _t->on_UpperZoomBoundary_valueChanged(); break;
        case 11: _t->on_DrawBackground_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->on_BackgroundColor_clicked(); break;
        case 13: _t->on_ProportionalBackground_valueChanged(); break;
        case 14: _t->on_FixedBackground_valueChanged(); break;
        case 15: _t->on_BackgroundInterior_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->on_BackgroundExterior_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->on_DrawForeground_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->on_DrawFillIcon_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->on_ForegroundColor_clicked(); break;
        case 20: _t->on_ProportionalForeground_valueChanged(); break;
        case 21: _t->on_FixedForeground_valueChanged(); break;
        case 22: _t->on_ForegroundDashed_clicked(); break;
        case 23: _t->on_ForegroundDashOn_valueChanged(); break;
        case 24: _t->on_ForegroundDashOff_valueChanged(); break;
        case 25: _t->on_DrawTrafficDirectionMarks_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->on_TrafficDirectionMarksColor_clicked(); break;
        case 27: _t->on_DrawTouchup_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->on_TouchupColor_clicked(); break;
        case 29: _t->on_ProportionalTouchup_valueChanged(); break;
        case 30: _t->on_FixedTouchup_valueChanged(); break;
        case 31: _t->on_TouchupDashed_clicked(); break;
        case 32: _t->on_TouchupDashOn_valueChanged(); break;
        case 33: _t->on_TouchupDashOff_valueChanged(); break;
        case 34: _t->on_DrawFill_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 35: _t->on_FillColor_clicked(); break;
        case 36: _t->on_DrawIcon_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 37: _t->on_IconName_textEdited(); break;
        case 38: _t->on_ProportionalIcon_valueChanged(); break;
        case 39: _t->on_FixedIcon_valueChanged(); break;
        case 40: _t->on_DrawLabel_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 41: _t->on_LabelColor_clicked(); break;
        case 42: _t->on_LabelTag_textEdited(); break;
        case 43: _t->on_ProportionalLabel_valueChanged(); break;
        case 44: _t->on_FixedLabel_valueChanged(); break;
        case 45: _t->on_DrawLabelBackground_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 46: _t->on_LabelBackgroundlColor_clicked(); break;
        case 47: _t->on_LabelFont_currentFontChanged((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 48: _t->on_LabelBackgroundTag_textEdited(); break;
        case 49: _t->on_AddButton_clicked(); break;
        case 50: _t->on_RemoveButton_clicked(); break;
        case 51: _t->on_DuplicateButton_clicked(); break;
        case 52: _t->on_btUp_clicked(); break;
        case 53: _t->on_btDown_clicked(); break;
        case 54: _t->on_buttonBox_clicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 55: _t->on_LabelHalo_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 56: _t->on_LabelArea_clicked((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 57: _t->on_edFilter_textChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 58: _t->on_btSelectorHelper_clicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PaintStyleEditor::*)(GlobalPainter * , QList<Painter> * )>(_a, &PaintStyleEditor::stylesApplied, 0))
            return;
    }
}

const QMetaObject *PaintStyleEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PaintStyleEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16PaintStyleEditorE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PaintStyleEditor"))
        return static_cast< Ui::PaintStyleEditor*>(this);
    return QDialog::qt_metacast(_clname);
}

int PaintStyleEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 59)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 59;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 59)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 59;
    }
    return _id;
}

// SIGNAL 0
void PaintStyleEditor::stylesApplied(GlobalPainter * _t1, QList<Painter> * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}
QT_WARNING_POP
