/********************************************************************************
** Form generated from reading UI file 'CreateRoundaboutDock.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREATEROUNDABOUTDOCK_H
#define UI_CREATEROUNDABOUTDOCK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_CreateRoundaboutDock
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *precisionLabel;
    QLabel *label;
    QDoubleSpinBox *precision;
    QComboBox *type;
    QComboBox *drivingSide;
    QLabel *label_2;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *CreateRoundaboutDock)
    {
        if (CreateRoundaboutDock->objectName().isEmpty())
            CreateRoundaboutDock->setObjectName("CreateRoundaboutDock");
        CreateRoundaboutDock->resize(364, 142);
        vboxLayout = new QVBoxLayout(CreateRoundaboutDock);
#ifndef Q_OS_MAC
        vboxLayout->setSpacing(6);
#endif
#ifndef Q_OS_MAC
        vboxLayout->setContentsMargins(9, 9, 9, 9);
#endif
        vboxLayout->setObjectName("vboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        precisionLabel = new QLabel(CreateRoundaboutDock);
        precisionLabel->setObjectName("precisionLabel");

        gridLayout->addWidget(precisionLabel, 3, 0, 1, 1);

        label = new QLabel(CreateRoundaboutDock);
        label->setObjectName("label");

        gridLayout->addWidget(label, 1, 0, 1, 1);

        precision = new QDoubleSpinBox(CreateRoundaboutDock);
        precision->setObjectName("precision");
        precision->setDecimals(1);
        precision->setMinimum(0.100000000000000);
        precision->setValue(3.000000000000000);

        gridLayout->addWidget(precision, 3, 1, 1, 1);

        type = new QComboBox(CreateRoundaboutDock);
        type->addItem(QString());
        type->addItem(QString());
        type->setObjectName("type");

        gridLayout->addWidget(type, 1, 1, 1, 1);

        drivingSide = new QComboBox(CreateRoundaboutDock);
        drivingSide->addItem(QString());
        drivingSide->addItem(QString());
        drivingSide->setObjectName("drivingSide");

        gridLayout->addWidget(drivingSide, 2, 1, 1, 1);

        label_2 = new QLabel(CreateRoundaboutDock);
        label_2->setObjectName("label_2");

        gridLayout->addWidget(label_2, 2, 0, 1, 1);


        vboxLayout->addLayout(gridLayout);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem);


        retranslateUi(CreateRoundaboutDock);

        QMetaObject::connectSlotsByName(CreateRoundaboutDock);
    } // setupUi

    void retranslateUi(QWidget *CreateRoundaboutDock)
    {
        CreateRoundaboutDock->setWindowTitle(QCoreApplication::translate("CreateRoundaboutDock", "Form", nullptr));
        precisionLabel->setText(QCoreApplication::translate("CreateRoundaboutDock", "Precision (meters/segment)", nullptr));
        label->setText(QCoreApplication::translate("CreateRoundaboutDock", "Type", nullptr));
        type->setItemText(0, QCoreApplication::translate("CreateRoundaboutDock", "Roundabout", nullptr));
        type->setItemText(1, QCoreApplication::translate("CreateRoundaboutDock", "Plain circle", nullptr));

        drivingSide->setItemText(0, QCoreApplication::translate("CreateRoundaboutDock", "Left", nullptr));
        drivingSide->setItemText(1, QCoreApplication::translate("CreateRoundaboutDock", "Right", nullptr));

        label_2->setText(QCoreApplication::translate("CreateRoundaboutDock", "Driving side", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CreateRoundaboutDock: public Ui_CreateRoundaboutDock {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CREATEROUNDABOUTDOCK_H
